/**
 * Generated by Apache Royale Compiler from mx/binding/BindingManager.as
 * mx.binding.BindingManager
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.binding.BindingManager');

goog.require('mx.binding.Binding');
goog.require('mx.binding.IBindingClient');
goog.require('mx.core.mx_internal');
goog.require('org.apache.royale.utils.Language');
goog.require('XML');



/**
 *  @asprivate
 *  BindingManager has only static methods.
 *  We don't create instances of BindingManager.
 * @constructor
 */
mx.binding.BindingManager = function() {
  ;
};


/**
 *  Store a Binding for the destination relative to the passed in document.
 *  We don't hold a list of bindings per destination
 *  even though it is possible to have multiple.
 *  The reason is that when we refresh the last binding will
 *  always win anyway, so why execute the ones that will lose.
 *
 *  @asparam document The document that this binding relates to.
 *
 *  @asparam destStr The destination field of this binding.
 *
 *  @asparam b The binding itself.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} document
 * @param {string} destStr
 * @param {mx.binding.Binding} b
 */
mx.binding.BindingManager.addBinding = function(document, destStr, b) {
  if (!document._bindingsByDestination) {
    document._bindingsByDestination = {};
    document._bindingsBeginWithWord = {};
  }
  document._bindingsByDestination[destStr] = b;
  document._bindingsBeginWithWord[mx.binding.BindingManager.getFirstWord(destStr)] = true;
};


/**
 *  Set isEnabled for all bindings associated with a document.
 *
 *  @asparam document The document that contains the bindings.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} document
 * @param {boolean} isEnabled
 */
mx.binding.BindingManager.setEnabled = function(document, isEnabled) {
  if (org.apache.royale.utils.Language.is(document, mx.binding.IBindingClient) && document._bindings) {
    var /** @type {Array} */ bindings = org.apache.royale.utils.Language.as(document._bindings, Array);
    for (var /** @type {number} */ i = 0; i < bindings.length; i++) {
      var /** @type {mx.binding.Binding} */ binding = bindings[i];
      binding.http_$$www_adobe_com$2006$flex$mx$internal__isEnabled = isEnabled;
    }
  }
};


/**
 *  Execute all bindings that bind into the specified object.
 *
 *  @asparam document The document that this binding relates to.
 *
 *  @asparam destStr The destination field that needs to be refreshed.
 *
 *  @asparam destObj The actual destination object
 *  (used for RepeatableBinding).
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} document
 * @param {string} destStr
 * @param {Object} destObj
 */
mx.binding.BindingManager.executeBindings = function(document, destStr, destObj) {
  if (!destStr || (destStr == ""))
    return;
  if (document && (org.apache.royale.utils.Language.is(document, mx.binding.IBindingClient) || document.hasOwnProperty("_bindingsByDestination")) && document._bindingsByDestination && document._bindingsBeginWithWord[mx.binding.BindingManager.getFirstWord(destStr)]) {
    for (var /** @type {string} */ binding in document._bindingsByDestination) {
      if (binding.charAt(0) == destStr.charAt(0)) {
        var /** @type {number} */ length = (destStr.length) >> 0;
        if ((length < binding.length && binding.indexOf(destStr) == 0 && (binding.charAt(length) == "." || binding.charAt(length) == "[")) || binding == destStr) {
          document._bindingsByDestination[binding].execute(destObj);
        }
      }
    }
  }
};


/**
 *  Enable or disable all bindings that bind into the specified object
 *  and match the input destStr.
 *
 *  @asparam document The document that this binding relates to.
 *
 *  @asparam destStr The destination field that needs to be refreshed.
 * 
 *  @asparam enable If true enables the specified binding(s), otherwise
 *  disables.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10
 *  @playerversion AIR 2/5
 *  @productversion Flex 4.5
 * @nocollapse
 * @param {Object} document
 * @param {string} destStr
 * @param {boolean=} enable
 */
mx.binding.BindingManager.enableBindings = function(document, destStr, enable) {
  enable = typeof enable !== 'undefined' ? enable : true;
  if (!destStr || (destStr == ""))
    return;
  if (document && (org.apache.royale.utils.Language.is(document, mx.binding.IBindingClient) || document.hasOwnProperty("_bindingsByDestination")) && document._bindingsByDestination && document._bindingsBeginWithWord[mx.binding.BindingManager.getFirstWord(destStr)]) {
    for (var /** @type {string} */ binding in document._bindingsByDestination) {
      if (binding.charAt(0) == destStr.charAt(0)) {
        if (binding.indexOf(destStr + ".") == 0 || binding.indexOf(destStr + "[") == 0 || binding == destStr) {
          document._bindingsByDestination[binding].isEnabled = enable;
        }
      }
    }
  }
};


/**
 *  @asprivate
 * @private
 * @param {string} destStr
 * @return {string}
 */
mx.binding.BindingManager.getFirstWord = function(destStr) {
  var /** @type {number} */ indexPeriod = (destStr.indexOf(".")) >> 0;
  var /** @type {number} */ indexBracket = (destStr.indexOf("[")) >> 0;
  if (indexPeriod == indexBracket)
    return destStr;
  var /** @type {number} */ minIndex = (Math.min(indexPeriod, indexBracket)) >> 0;
  if (minIndex == -1)
    minIndex = (Math.max(indexPeriod, indexBracket)) >> 0;
  return destStr.substr(0, minIndex);
};


/**
 * @package
 * @nocollapse
 * @type {Object}
 */
mx.binding.BindingManager.debugDestinationStrings = {};


/**
 *  Enables debugging output for the Binding or Bindings with a matching
 *  destination string.
 *
 *  @asparam destinationString The Binding's destination string.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {string} destinationString
 */
mx.binding.BindingManager.debugBinding = function(destinationString) {
  mx.binding.BindingManager.debugDestinationStrings[destinationString] = true;
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.binding.BindingManager.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'BindingManager', qName: 'mx.binding.BindingManager', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.binding.BindingManager.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    methods: function () {
      return {
        '|addBinding': { type: 'void', declaredBy: 'mx.binding.BindingManager', parameters: function () { return [ 'Object', false ,'String', false ,'mx.binding.Binding', false ]; }},
        '|setEnabled': { type: 'void', declaredBy: 'mx.binding.BindingManager', parameters: function () { return [ 'Object', false ,'Boolean', false ]; }},
        '|executeBindings': { type: 'void', declaredBy: 'mx.binding.BindingManager', parameters: function () { return [ 'Object', false ,'String', false ,'Object', false ]; }},
        '|enableBindings': { type: 'void', declaredBy: 'mx.binding.BindingManager', parameters: function () { return [ 'Object', false ,'String', false ,'Boolean', true ]; }},
        '|debugBinding': { type: 'void', declaredBy: 'mx.binding.BindingManager', parameters: function () { return [ 'String', false ]; }},
        'BindingManager': { type: '', declaredBy: 'mx.binding.BindingManager'}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.binding.BindingManager.prototype.ROYALE_COMPILE_FLAGS = 10;