/**
 * Generated by Apache Royale Compiler from mx/binding/utils/BindingUtils.as
 * mx.binding.utils.BindingUtils
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.binding.utils.BindingUtils');

goog.require('mx.binding.utils.ChangeWatcher');
goog.require('XML');



/**
 * @constructor
 */
mx.binding.utils.BindingUtils = function() {
};


/**
 *  Binds a public property, <code>prop</code> on the <code>site</code>
 *  Object, to a bindable property or property chain. 
 *  If a ChangeWatcher instance is successfully created, <code>prop</code>
 *  is initialized to the current value of <code>chain</code>.
 * 
 *  @asparam site The Object defining the property to be bound
 *  to <code>chain</code>.
 * 
 *  @asparam prop The name of the public property defined in the
 *  <code>site</code> Object to be bound. 
 *  The property will receive the current value of <code>chain</code>, 
 *  when the value of <code>chain</code> changes. 
 *
 *  @asparam host The object that hosts the property or property chain
 *  to be watched. 
 *  The <code>host</code> maintains a list of <code>sites</code> to update 
 *  when <code>prop</code> changes.
 *
 *  @asparam chain A value specifying the property or chain to be watched.
 *  Legal values are:
 *  <ul>
 *    <li>String containing the name of a public bindable property
 *    of the host object.</li>
 * 
 *    <li>An Object in the form: 
 *    <code>{ name: <i>property name</i>, getter: function(host) { return host[<i>property name</i>] } }</code>. 
 *    This Object must contain the name of, and a getter function for, 
 *    a public bindable property of the host object.</li>
 * 
 *    <li>A non-empty Array containing a combination of the first two
 *    options that represents a chain of bindable properties accessible
 *    from the host. 
 *    For example, to bind the property <code>host.a.b.c</code>, 
 *    call the method as:
 *    <code>bindProperty(site, prop, host, ["a","b","c"])</code>.</li>
 *  </ul>
 *
 *  <p>Note: The property or properties named in the <code>chain</code> argument
 *  must be public, because the <code>describeType()</code> method suppresses all information
 *  about non-public properties, including the bindability metadata
 *  that ChangeWatcher scans to find the change events that are exposed
 *  for a given property.
 *  However, the getter function supplied when using the <code>{ name, getter }</code>
 *  argument form described above can be used to associate an arbitrary
 *  computed value with the named (public) property.</p>
 *
 *  @asparam commitOnly Set to <code>true</code> if the handler
 *  should be called only on committing change events;
 *  set to <code>false</code> if the handler should be called
 *  on both committing and non-committing change events. 
 *  Note: the presence of non-committing change events for a property
 *  is indicated by the <code>[NonCommittingChangeEvent(&lt;event-name&gt;)]</code>
 *  metadata tag. 
 *  Typically these tags are used to indicate fine-grained value changes, 
 *  such as modifications in a text field prior to confirmation.
 *
 *  @asparam useWeakReference Determines whether the reference to <code>site</code>
 *  is strong or weak. 
 *  A strong reference (the default) prevents <code>site</code> from being garbage-collected. 
 *  A weak reference does not. 
 *  Added for Flex 4.
 *
 *  @asreturn A ChangeWatcher instance, if at least one property name has
 *  been specified to the <code>chain</code> argument; null otherwise. 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} site
 * @param {string} prop
 * @param {Object} host
 * @param {Object} chain
 * @return {mx.binding.utils.ChangeWatcher}
 */
mx.binding.utils.BindingUtils.bindProperty = function(site, prop, host, chain) {
  var /** @type {mx.binding.utils.ChangeWatcher} */ w = mx.binding.utils.ChangeWatcher.watch(host, chain, null);
  if (w != null) {
    var /** @type {Function} */ assign = function(event) {
      site[prop] = w.getValue();
    };
    w.setHandler(assign);
    assign(null);
  }
  return w;
};


/**
 *  Binds a setter function, <code>setter</code>, to a bindable property 
 *  or property chain.
 *  If a ChangeWatcher instance is successfully created, 
 *  the setter function is invoked. 
 *  The setter must have the following function signature:
 *
 *  <pre>
 *  function mySetterFunction(object:Object):void {
 *      //Do whatever you want with the value of the bound property.
 *  }</pre> 
 *
 *  <p>where <code>object</code> contains the
 *  current value of <code>chain</code>.</p>
 *
 *  @asparam setter Setter method to invoke with an argument of the current
 *  value of <code>chain</code> when that value changes.
 *
 *  @asparam host The host of the property. 
 *  See the <code>bindProperty()</code> method for more information.
 *  The <code>host</code> maintains a list of <code>setters</code> to update 
 *  when <code>prop</code> changes.
 *
 *  @asparam chain The name of the property, or property chain.
 *  See the <code>bindProperty()</code> method for more information.
 *
 *  @asparam commitOnly Set to <code>true</code> if the handler should be
 *  called only on committing change events.
 *  See the <code>bindProperty()</code> method for more information.
 *
 *  @asparam useWeakReference Determines whether the reference to <code>setter</code>
 *  is strong or weak. 
 *  A strong reference (the default) prevents <code>setter</code> from being garbage-collected. 
 *  A weak reference does not. 
 *  Added for Flex 4.
 *
 *  @asreturn A ChangeWatcher instance, if at least one property name
 *  has been  specified to the <code>chain</code> argument; null otherwise. 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Function} setter
 * @param {Object} host
 * @param {Object} chain
 * @return {mx.binding.utils.ChangeWatcher}
 */
mx.binding.utils.BindingUtils.bindSetter = function(setter, host, chain) {
  var /** @type {mx.binding.utils.ChangeWatcher} */ w = mx.binding.utils.ChangeWatcher.watch(host, chain, null);
  if (w != null) {
    var /** @type {Function} */ invoke = function(event) {
      setter(w.getValue());
    };
    w.setHandler(invoke);
    invoke(null);
  }
  return w;
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.binding.utils.BindingUtils.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'BindingUtils', qName: 'mx.binding.utils.BindingUtils', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.binding.utils.BindingUtils.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    methods: function () {
      return {
        '|bindProperty': { type: 'mx.binding.utils.ChangeWatcher', declaredBy: 'mx.binding.utils.BindingUtils', parameters: function () { return [ 'Object', false ,'String', false ,'Object', false ,'Object', false ]; }},
        '|bindSetter': { type: 'mx.binding.utils.ChangeWatcher', declaredBy: 'mx.binding.utils.BindingUtils', parameters: function () { return [ 'Function', false ,'Object', false ,'Object', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.binding.utils.BindingUtils.prototype.ROYALE_COMPILE_FLAGS = 10;