/**
 * Generated by Apache Royale Compiler from mx/binding/utils/ChangeWatcher.as
 * mx.binding.utils.ChangeWatcher
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.binding.utils.ChangeWatcher');

goog.require('mx.binding.BindabilityInfo');
goog.require('mx.events.PropertyChangeEvent');
goog.require('org.apache.royale.events.Event');
goog.require('org.apache.royale.events.IEventDispatcher');
goog.require('org.apache.royale.events.ValueChangeEvent');
goog.require('org.apache.royale.utils.Language');
goog.require('XML');



/**
 *  Constructor.
 *  Not for public use. This method is called only from the <code>watch()</code> method.
 *  See the <code>watch()</code> method for parameter usage.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @param {Object} access
 * @param {Function} handler
 * @param {mx.binding.utils.ChangeWatcher=} next
 */
mx.binding.utils.ChangeWatcher = function(access, handler, next) {
  next = typeof next !== 'undefined' ? next : null;
  ;
  this.mx_binding_utils_ChangeWatcher_host = null;
  this.mx_binding_utils_ChangeWatcher_name = org.apache.royale.utils.Language.string(org.apache.royale.utils.Language.is(access, String) ? org.apache.royale.utils.Language.as(access, String) : access.name);
  this.mx_binding_utils_ChangeWatcher_getter = org.apache.royale.utils.Language.is(access, String) ? null : access.getter;
  this.mx_binding_utils_ChangeWatcher_handler = handler;
  this.mx_binding_utils_ChangeWatcher_next = next;
  this.mx_binding_utils_ChangeWatcher_events = {};
  this.mx_binding_utils_ChangeWatcher_isExecuting = false;
};


/**
 *  Creates and starts a ChangeWatcher instance.
 *  A single ChangeWatcher instance can watch one property,
 *  or a property chain.
 *  A property chain is a sequence of properties accessible from
 *  a host object.
 *  For example, the expression
 *  <code>obj.a.b.c</code> contains the property chain (a, b, c).
 *
 *  @asparam host The object that hosts the property or property chain
 *  to be watched.
 *  You can use the use the <code>reset()</code> method to change
 *  the value of the <code>host</code> argument after creating
 *  the ChangeWatcher instance.
 *  The <code>host</code> maintains a list of <code>handlers</code> to invoke 
 *  when <code>prop</code> changes.
 *
 *  @asparam chain A value specifying the property or chain to be watched.
 *  Legal values are:
 *  <ul>
 *    <li>A String containing the name of a public bindable property
 *    of the host object.</li>
 *
 *    <li>An Object in the form:
 *    <code>{ name: <i>property name</i>, getter: function(host) { return host[name] } }</code>.
 *    The Object contains the name of a public bindable property,
 *    and a function which serves as a getter for that property.</li>
 *
 *    <li>A non-empty Array containing any combination
 *    of the first two options.
 *    This represents a chain of bindable properties
 *    accessible from the host.
 *    For example, to watch the property <code>host.a.b.c</code>,
 *    call the method as: <code>watch(host, ["a","b","c"], ...)</code>.</li>
 *  </ul>
 *
 *  <p>Note: The property or properties named in the <code>chain</code> argument
 *  must be public, because the <code>describeType()</code> method suppresses all information
 *  about non-public properties, including the bindability metadata
 *  that ChangeWatcher scans to find the change events that are exposed
 *  for a given property.
 *  However, the getter function supplied when using the <code>{ name, getter }</code>
 *  argument form described above can be used to associate an arbitrary
 *  computed value with the named (public) property.</p>
 *
 *  @asparam handler An event handler function called when the value of the
 *  watched property (or any property in a watched chain) is modified.
 *  The modification is signaled when any host object in the watcher
 *  chain dispatches the event that has been specified in that host object's
 *  <code>[Bindable]</code> metadata tag for the corresponding watched property.
 *  The default event is named <code>propertyChange</code>.
 *
 *  <p>The event object dispatched by the bindable property is passed
 *  to this handler function without modification.
 *  By default, Flex dispatches an event object of type PropertyChangeEvent.
 *  However, you can define your own event type when you use the
 *  <code>[Bindable]</code> metadata tag to define a bindable property.</p>
 *
 *  @asparam commitOnly Set to <code>true</code> if the handler should be
 *  called only on committing change events;
 *  set to <code>false</code> if the handler should be called on both
 *  committing and non-committing change events.
 *  Note: the presence of non-committing change events for a property is
 *  indicated by the <code>[NonCommittingChangeEvent(&lt;event-name&gt;)]</code> metadata tag.
 *  Typically these tags are used to indicate fine-grained value changes,
 *  such as modifications in a text field prior to confirmation.
 *
 *  @asparam useWeakReference (default = false) Determines whether
 *  the reference to <code>handler</code> is strong or weak. A strong
 *  reference (the default) prevents <code>handler</code> from being
 *  garbage-collected. A weak reference does not.
 *
 *  @asreturn The ChangeWatcher instance, if at least one property name has
 *  been specified to the <code>chain</code> argument; null otherwise.
 *  Note that the returned watcher is not guaranteed to have successfully
 *  discovered and attached itself to change events, since none may have
 *  been exposed on the given property or chain by the host.
 *  You can use the <code>isWatching()</code> method to determine the
 *  watcher's state.
 *
 *  @see mx.events.PropertyChangeEvent
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} host
 * @param {Object} chain
 * @param {Function} handler
 * @return {mx.binding.utils.ChangeWatcher}
 */
mx.binding.utils.ChangeWatcher.watch = function(host, chain, handler) {
  if (!org.apache.royale.utils.Language.is(chain, Array))
    chain = [chain];
  if (chain.length > 0) {
    var /** @type {mx.binding.utils.ChangeWatcher} */ w = new mx.binding.utils.ChangeWatcher(chain[0], handler, mx.binding.utils.ChangeWatcher.watch(null, chain.slice(1), handler));
    w.reset(host);
    return w;
  } else {
    return null;
  }
};


/**
 *  Lets you determine if the host exposes a data-binding event
 *  on the property. 
 * 
 *  <p>NOTE: Property chains are not supported by the <code>canWatch()</code> method.
 * They are supported by the <code>watch()</code> method.</p>
 *
 *  @asparam host The host of the property.
 *  See the <code>watch()</code> method for more information.
 *
 *  @asparam name The name of the property.
 *  See the <code>watch()</code> method for more information.
 *
 *  @asparam commitOnly Set to <code>true</code> if the handler
 *  should be called only on committing change events.
 *  See the <code>watch()</code> method for more information.
 *
 *  @asreturn <code>true</code> if <code>host</code> exposes
 *  any change events on <code>name</code>.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} host
 * @param {string} name
 * @return {boolean}
 */
mx.binding.utils.ChangeWatcher.canWatch = function(host, name) {
  return !mx.binding.utils.ChangeWatcher.isEmpty(mx.binding.utils.ChangeWatcher.getEvents(host, name));
};


/**
 *  Returns all binding events for a bindable property in the host object.
 *
 *  @asparam host The host of the property.
 *  See the <code>watch()</code> method for more information.
 *
 *  @asparam name The name of the property, or property chain.
 *  See the <code>watch()</code> method for more information.
 *
 *  @asparam commitOnly Controls inclusion of non-committing
 *  change events in the returned value.
 *
 *  @asreturn Object of the form <code>{ eventName: isCommitting, ... }</code>
 *  containing all change events for the property.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {Object} host
 * @param {string} name
 * @return {Object}
 */
mx.binding.utils.ChangeWatcher.getEvents = function(host, name) {
  if (org.apache.royale.utils.Language.is(host, org.apache.royale.events.IEventDispatcher)) {
    var /** @type {Object} */ allEvents = mx.binding.BindabilityInfo.getCachedInfo(host).getChangeEvents(name);
    return allEvents;
  } else {
    return {};
  }
};


/**
 *  Lets you determine if an Object has any properties.
 *
 *  @asparam obj to inspect.
 *
 *  @asreturn <code>true</code> if Object has no properties.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @private
 * @param {Object} obj
 * @return {boolean}
 */
mx.binding.utils.ChangeWatcher.isEmpty = function(obj) {
  for (var /** @type {string} */ p in obj) {
    return false;
  }
  return true;
};


/**
 * @private
 * @type {Object}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_host;


/**
 * @private
 * @type {string}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_name;


/**
 * @private
 * @type {Function}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_getter;


/**
 * @private
 * @type {Function}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_handler;


/**
 * @private
 * @type {mx.binding.utils.ChangeWatcher}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_next;


/**
 * @private
 * @type {Object}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_events;


/**
 * @private
 * @type {boolean}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_isExecuting;


/**
 * @type {boolean}
 */
mx.binding.utils.ChangeWatcher.prototype.useWeakReference;


/**
 *  Detaches this ChangeWatcher instance, and its handler function,
 *  from the current host.
 *  You can use the <code>reset()</code> method to reattach
 *  the ChangeWatcher instance, or watch the same property
 *  or chain on a different host object.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 */
mx.binding.utils.ChangeWatcher.prototype.unwatch = function() {
  this.reset(null);
};


/**
 *  Retrieves the current value of the watched property or property chain,
 *  or null if the host object is null.
 *  For example:
 *  <pre>
 *  watch(obj, ["a","b","c"], ...).getValue() === obj.a.b.c
 *  </pre>
 *
 *  @asreturn The current value of the watched property or property chain.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @return {Object}
 */
mx.binding.utils.ChangeWatcher.prototype.getValue = function() {
  return this.mx_binding_utils_ChangeWatcher_host == null ? null : this.mx_binding_utils_ChangeWatcher_next == null ? this.mx_binding_utils_ChangeWatcher_getHostPropertyValue() : this.mx_binding_utils_ChangeWatcher_next.getValue();
};


/**
 *  Sets the handler function.
 *
 *  @asparam handler The handler function. This argument must not be null.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {Function} handler
 */
mx.binding.utils.ChangeWatcher.prototype.setHandler = function(handler) {
  this.mx_binding_utils_ChangeWatcher_handler = handler;
  if (this.mx_binding_utils_ChangeWatcher_next)
    this.mx_binding_utils_ChangeWatcher_next.setHandler(handler);
};


/**
 *  Returns <code>true</code> if each watcher in the chain is attached
 *  to at least one change event.
 *  Note that the <code>isWatching()</code> method
 *  varies with host, since different hosts may expose different change
 *  events for the watcher's chosen property.
 *
 *  @asreturn <code>true</code> if each watcher in the chain is attached
 *  to at least one change event.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @return {boolean}
 */
mx.binding.utils.ChangeWatcher.prototype.isWatching = function() {
  return !mx.binding.utils.ChangeWatcher.isEmpty(this.mx_binding_utils_ChangeWatcher_events) && (this.mx_binding_utils_ChangeWatcher_next == null || this.mx_binding_utils_ChangeWatcher_next.isWatching());
};


/**
 *  Resets this ChangeWatcher instance to use a new host object.
 *  You can call this method to reuse a watcher instance
 *  on a different host.
 *
 *  @asparam newHost The new host of the property.
 *  See the <code>watch()</code> method for more information.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {Object} newHost
 */
mx.binding.utils.ChangeWatcher.prototype.reset = function(newHost) {
  var /** @type {string} */ p;
  if (this.mx_binding_utils_ChangeWatcher_host != null) {
    for (p in this.mx_binding_utils_ChangeWatcher_events) {
      this.mx_binding_utils_ChangeWatcher_host.removeEventListener(p, org.apache.royale.utils.Language.closure(this.mx_binding_utils_ChangeWatcher_wrapHandler, this, 'mx_binding_utils_ChangeWatcher_wrapHandler'));
    }
    if (newHost == null)
      this.mx_binding_utils_ChangeWatcher_events = {};
  }
  this.mx_binding_utils_ChangeWatcher_host = newHost;
  if (this.mx_binding_utils_ChangeWatcher_host != null) {
    this.mx_binding_utils_ChangeWatcher_events = mx.binding.utils.ChangeWatcher.getEvents(this.mx_binding_utils_ChangeWatcher_host, this.mx_binding_utils_ChangeWatcher_name);
    for (p in this.mx_binding_utils_ChangeWatcher_events) {
      this.mx_binding_utils_ChangeWatcher_host.addEventListener(p, org.apache.royale.utils.Language.closure(this.mx_binding_utils_ChangeWatcher_wrapHandler, this, 'mx_binding_utils_ChangeWatcher_wrapHandler'), false);
    }
  }
  if (this.mx_binding_utils_ChangeWatcher_next)
    this.mx_binding_utils_ChangeWatcher_next.reset(this.mx_binding_utils_ChangeWatcher_getHostPropertyValue());
};


/**
 *  @asprivate
 *  Retrieves the current value of the root watched property,
 *  or null if host is null.
 *  I.e. watch(obj, ["a","b","c"], ...).getHostPropertyValue() === obj.a
 * @private
 * @return {Object}
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_getHostPropertyValue = function() {
  return this.mx_binding_utils_ChangeWatcher_host == null ? null : this.mx_binding_utils_ChangeWatcher_getter != null ? this.mx_binding_utils_ChangeWatcher_getter(this.mx_binding_utils_ChangeWatcher_host) : this.mx_binding_utils_ChangeWatcher_host[this.mx_binding_utils_ChangeWatcher_name];
};


/**
 *  @asprivate
 *  Listener for change events.
 *  Resets chained watchers and calls user-supplied handler.
 *
 * @private
 * @param {org.apache.royale.events.Event} event
 */
mx.binding.utils.ChangeWatcher.prototype.mx_binding_utils_ChangeWatcher_wrapHandler = function(event) {
  if (!this.mx_binding_utils_ChangeWatcher_isExecuting) {
    try {
      this.mx_binding_utils_ChangeWatcher_isExecuting = true;
      if (this.mx_binding_utils_ChangeWatcher_next)
        this.mx_binding_utils_ChangeWatcher_next.reset(this.mx_binding_utils_ChangeWatcher_getHostPropertyValue());
      if (org.apache.royale.utils.Language.is(event, org.apache.royale.events.ValueChangeEvent)) {
        if (org.apache.royale.utils.Language.as(event, org.apache.royale.events.ValueChangeEvent).propertyName == this.mx_binding_utils_ChangeWatcher_name)
          this.mx_binding_utils_ChangeWatcher_handler(event);
      } else if (org.apache.royale.utils.Language.is(event, mx.events.PropertyChangeEvent)) {
        if (org.apache.royale.utils.Language.as(event, mx.events.PropertyChangeEvent).property == this.mx_binding_utils_ChangeWatcher_name)
          this.mx_binding_utils_ChangeWatcher_handler(event);
      } else {
        this.mx_binding_utils_ChangeWatcher_handler(event);
      }
    } finally {
      this.mx_binding_utils_ChangeWatcher_isExecuting = false;
    }
  }
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.binding.utils.ChangeWatcher.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'ChangeWatcher', qName: 'mx.binding.utils.ChangeWatcher', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.binding.utils.ChangeWatcher.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'useWeakReference': { type: 'Boolean', get_set: function (/** mx.binding.utils.ChangeWatcher */ inst, /** * */ v) {return v !== undefined ? inst.useWeakReference = v : inst.useWeakReference;}}
      };
    },
    methods: function () {
      return {
        '|watch': { type: 'mx.binding.utils.ChangeWatcher', declaredBy: 'mx.binding.utils.ChangeWatcher', parameters: function () { return [ 'Object', false ,'Object', false ,'Function', false ]; }},
        '|canWatch': { type: 'Boolean', declaredBy: 'mx.binding.utils.ChangeWatcher', parameters: function () { return [ 'Object', false ,'String', false ]; }},
        '|getEvents': { type: 'Object', declaredBy: 'mx.binding.utils.ChangeWatcher', parameters: function () { return [ 'Object', false ,'String', false ]; }},
        'ChangeWatcher': { type: '', declaredBy: 'mx.binding.utils.ChangeWatcher', parameters: function () { return [ 'Object', false ,'Function', false ,'mx.binding.utils.ChangeWatcher', true ]; }},
        'unwatch': { type: 'void', declaredBy: 'mx.binding.utils.ChangeWatcher'},
        'getValue': { type: 'Object', declaredBy: 'mx.binding.utils.ChangeWatcher'},
        'setHandler': { type: 'void', declaredBy: 'mx.binding.utils.ChangeWatcher', parameters: function () { return [ 'Function', false ]; }},
        'isWatching': { type: 'Boolean', declaredBy: 'mx.binding.utils.ChangeWatcher'},
        'reset': { type: 'void', declaredBy: 'mx.binding.utils.ChangeWatcher', parameters: function () { return [ 'Object', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.binding.utils.ChangeWatcher.prototype.ROYALE_COMPILE_FLAGS = 10;