/**
 * Generated by Apache Royale Compiler from mx/formatters/DateFormatter.as
 * mx.formatters.DateFormatter
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.formatters.DateFormatter');

goog.require('mx.core.mx_internal');
goog.require('mx.formatters.DateBase');
goog.require('mx.formatters.Formatter');
goog.require('mx.formatters.StringFormatter');
goog.require('org.apache.royale.utils.Language');
goog.require('XML');



/**
 *  Constructor.
 
 * 	@asparam formatString Date format pattern is set to this DateFormatter.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Royale 0.9.3
 * @constructor
 * @extends {mx.formatters.Formatter}
 * @param {string=} formatString
 */
mx.formatters.DateFormatter = function(formatString) {
  formatString = typeof formatString !== 'undefined' ? formatString : null;
  mx.formatters.DateFormatter.base(this, 'constructor');
  this.formatString = formatString;
};
goog.inherits(mx.formatters.DateFormatter, mx.formatters.Formatter);


/**
 * @private
 * @const
 * @type {string}
 */
mx.formatters.DateFormatter.VALID_PATTERN_CHARS = "Y,M,D,A,E,H,J,K,L,N,S,Q,O,Z";


/**
 *  Converts a date that is formatted as a String into a Date object.
 *  Month and day names must match the names in mx.formatters.DateBase.
 *
 *  The hour value in the String must be between 0 and 23, inclusive. 
 *  The minutes and seconds value must be between 0 and 59, inclusive.
 *  The following example uses this method to create a Date object:
 *
 *  <pre>
 *  var myDate:Date = DateFormatter.parseDateString("2009-12-02 23:45:30"); </pre>
 * 
 *  The optional format property is use to work out which is likly to be encountered
 *  first a month or a date of the month for date where it may not be obvious which
 *  comes first.
 *  
 *  @see mx.formatters.DateBase
 * 
 *  @asparam str Date that is formatted as a String. 
 *
 *  @asreturn Date object.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Royale 0.9.3
 * @nocollapse
 * @param {string} str
 * @param {string=} format
 * @return {Date}
 */
mx.formatters.DateFormatter.parseDateString = function(str, format) {
  format = typeof format !== 'undefined' ? format : null;
  if (!str || str == "")
    return null;
  var /** @type {number} */ year = -1;
  var /** @type {number} */ mon = -1;
  var /** @type {number} */ day = -1;
  var /** @type {number} */ hour = -1;
  var /** @type {number} */ min = -1;
  var /** @type {number} */ sec = -1;
  var /** @type {number} */ milli = -1;
  var /** @type {string} */ letter = "";
  var /** @type {Object} */ marker = 0;
  var /** @type {number} */ count = 0;
  var /** @type {number} */ len = 0;
  var /** @type {boolean} */ isPM = false;
  var /** @type {Object} */ punctuation = {};
  var /** @type {Object} */ ampm = {};
  punctuation["/"] = {general:true, date:true, time:false};
  punctuation[":"] = {general:true, date:false, time:true};
  punctuation[" "] = {general:true, date:true, time:false};
  punctuation["."] = {general:true, date:true, time:true};
  punctuation[","] = {general:true, date:true, time:false};
  punctuation["+"] = {general:true, date:false, time:false};
  punctuation["-"] = {general:true, date:true, time:false};
  punctuation["年"] = {general:true, date:true, time:false};
  punctuation["月"] = {general:true, date:true, time:false};
  punctuation["日"] = {general:true, date:true, time:false};
  punctuation["午"] = {general:false, date:false, time:true};
  punctuation["년"] = {general:true, date:true, time:false};
  punctuation["월"] = {general:true, date:true, time:false};
  punctuation["일"] = {general:true, date:true, time:false};
  ampm["PM"] = true;
  ampm["pm"] = true;
  ampm["㏘"] = true;
  ampm["μμ"] = true;
  ampm["午後"] = true;
  ampm["上午"] = true;
  ampm["오후"] = true;
  var /** @type {RegExp} */ timezoneRegEx = /(GMT|UTC)((\+|-)\d\d\d\d\u0020)?/ig;
  str = str.replace(timezoneRegEx, "");
  len = (str.length) >> 0;
  while (count < len) {
    letter = str.charAt(count);
    count++;
    if (letter <= " " || letter == ",")
      continue;
    if (punctuation.hasOwnProperty(letter) && punctuation[letter].general) {
      marker = letter;
      continue;
    }
    if (!("0" <= letter && letter <= "9" || punctuation.hasOwnProperty(letter) && punctuation[letter].general)) {
      var /** @type {string} */ word = letter;
      while (count < len) {
        letter = str.charAt(count);
        if ("0" <= letter && letter <= "9" || punctuation.hasOwnProperty(letter) && punctuation[letter].general) {
          break;
        }
        word += letter;
        count++;
      }
      var /** @type {number} */ n = (mx.formatters.DateBase.http_$$www_adobe_com$2006$flex$mx$internal__defaultStringKey.length) >> 0;
      for (var /** @type {number} */ i = 0; i < n; i++) {
        var /** @type {string} */ s = String(mx.formatters.DateBase.http_$$www_adobe_com$2006$flex$mx$internal__defaultStringKey[i]);
        if (s.toLowerCase() == word.toLowerCase() || s.toLowerCase().substr(0, 3) == word.toLowerCase()) {
          if (i == 13) {
            if (hour > 12 || hour < 1)
              break;
            else if (hour < 12)
              hour += 12;
          } else if (i == 12) {
            if (hour > 12 || hour < 1)
              break;
            else if (hour == 12)
              hour = 0;
          } else if (i < 12) {
            if (mon < 0)
              mon = i;
            else
              break;
          }
          break;
        }
      }
      marker = 0;
      if (ampm.hasOwnProperty(word)) {
        isPM = true;
        if (hour > 12)
          break;
        else if (hour >= 0 && hour < 12)
          hour += 12;
      }
    } else if ("0" <= letter && letter <= "9") {
      var /** @type {string} */ numbers = letter;
      while ("0" <= (letter = str.charAt(count)) && letter <= "9" && count < len) {
        numbers += letter;
        count++;
      }
      var /** @type {number} */ num = org.apache.royale.utils.Language._int(numbers);
      if (num >= 70 && !(punctuation.hasOwnProperty(letter) && punctuation[letter].time && sec >= 0)) {
        if (year != -1) {
          break;
        } else if (punctuation.hasOwnProperty(letter) && punctuation[letter].date || count >= len) {
          year = num;
        } else {
          break;
        }
      } else if (punctuation.hasOwnProperty(letter) && punctuation[letter].date) {
        var /** @type {boolean} */ monthFirst = year != -1;
        if (format)
          monthFirst = monthFirst || format.search("M") < format.search("D");
        if (monthFirst && mon < 0)
          mon = ((num - 1)) >> 0;
        else if (day < 0)
          day = num;
        else if (!monthFirst && mon < 0)
          mon = ((num - 1)) >> 0;
        else if (min < 0)
          min = num;
        else if (sec < 0)
          sec = num;
        else if (milli < 0)
          milli = num;
        else
          break;
      } else if (punctuation.hasOwnProperty(letter) && punctuation[letter].time) {
        if (hour < 0) {
          hour = num;
          if (isPM) {
            if (hour > 12)
              break;
            else if (hour >= 0 && hour < 12)
              hour += 12;
          }
        }
        else if (min < 0)
          min = num;
        else if (sec < 0)
          sec = num;
        else if (milli < 0)
          milli = num;
        else
          break;
      } else if (hour >= 0 && min < 0) {
        min = num;
      } else if (min >= 0 && sec < 0) {
        sec = num;
      } else if (sec >= 0 && milli < 0) {
        milli = num;
      } else if (day < 0) {
        day = num;
      } else if (year < 0 && mon >= 0 && day >= 0) {
        year = (2000 + num) >> 0;
      } else {
        break;
      }
      marker = 0;
    }
  }
  if (year < 0 || mon < 0 || mon > 11 || day < 1 || day > 31)
    return null;
  if (milli < 0)
    milli = 0;
  if (sec < 0)
    sec = 0;
  if (min < 0)
    min = 0;
  if (hour < 0)
    hour = 0;
  var /** @type {Date} */ newDate = new Date(year, mon, day, hour, min, sec, milli);
  if (day != newDate.getDate() || mon != newDate.getMonth())
    return null;
  return newDate;
};


/**
 * @private
 * @type {string}
 */
mx.formatters.DateFormatter.prototype.mx_formatters_DateFormatter__formatString;


/**
 *  Generates a date-formatted String from either a date-formatted String or a Date object. 
 *  The <code>formatString</code> property
 *  determines the format of the output String.
 *  If <code>value</code> cannot be formatted, return an empty String 
 *  and write a description of the error to the <code>error</code> property.
 *
 *  @asparam value Date to format. This can be a Date object,
 *  or a date-formatted String such as "Thursday, April 22, 2004".
 *
 *  @asreturn Formatted String. Empty if an error occurs. A description 
 *  of the error condition is written to the <code>error</code> property.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Royale 0.9.3
 * @override
 */
mx.formatters.DateFormatter.prototype.format = function(value) {
  if (this.error)
    this.error = null;
  if (!value || (org.apache.royale.utils.Language.is(value, String) && value == "")) {
    this.error = mx.formatters.Formatter.defaultInvalidValueError;
    return "";
  }
  if (org.apache.royale.utils.Language.is(value, String)) {
    value = mx.formatters.DateFormatter.parseDateString(String(value), this.formatString);
    if (!value) {
      this.error = mx.formatters.Formatter.defaultInvalidValueError;
      return "";
    }
  } else if (!org.apache.royale.utils.Language.is(value, Date)) {
    this.error = mx.formatters.Formatter.defaultInvalidValueError;
    return "";
  }
  var /** @type {string} */ letter;
  var /** @type {number} */ nTokens = 0;
  var /** @type {string} */ tokens = "";
  var /** @type {number} */ n = (this.formatString.length) >> 0;
  for (var /** @type {number} */ i = 0; i < n; i++) {
    letter = this.formatString.charAt(i);
    if (mx.formatters.DateFormatter.VALID_PATTERN_CHARS.indexOf(letter) != -1 && letter != ",") {
      nTokens++;
      if (tokens.indexOf(letter) == -1) {
        tokens += letter;
      } else {
        if (letter != this.formatString.charAt(Math.max(i - 1, 0))) {
          this.error = mx.formatters.Formatter.defaultInvalidFormatError;
          return "";
        }
      }
    }
  }
  if (nTokens < 1) {
    this.error = mx.formatters.Formatter.defaultInvalidFormatError;
    return "";
  }
  var /** @type {mx.formatters.StringFormatter} */ dataFormatter = new mx.formatters.StringFormatter(this.formatString, mx.formatters.DateFormatter.VALID_PATTERN_CHARS, mx.formatters.DateBase.http_$$www_adobe_com$2006$flex$mx$internal__extractTokenDate);
  return dataFormatter.formatValue(value);
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.formatters.DateFormatter.prototype.formatString;


mx.formatters.DateFormatter.prototype.get__formatString = function() {
  return this.mx_formatters_DateFormatter__formatString;
};


mx.formatters.DateFormatter.prototype.set__formatString = function(value) {
  this.mx_formatters_DateFormatter__formatString = value != null ? value : "MM/DD/YYYY";
};


Object.defineProperties(mx.formatters.DateFormatter.prototype, /** @lends {mx.formatters.DateFormatter.prototype} */ {
/**
 * @type {string}
 */
formatString: {
get: mx.formatters.DateFormatter.prototype.get__formatString,
set: mx.formatters.DateFormatter.prototype.set__formatString}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.formatters.DateFormatter.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'DateFormatter', qName: 'mx.formatters.DateFormatter', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.formatters.DateFormatter.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    accessors: function () {
      return {
        'formatString': { type: 'String', access: 'readwrite', declaredBy: 'mx.formatters.DateFormatter'}
      };
    },
    methods: function () {
      return {
        '|parseDateString': { type: 'Date', declaredBy: 'mx.formatters.DateFormatter', parameters: function () { return [ 'String', false ,'String', true ]; }},
        'DateFormatter': { type: '', declaredBy: 'mx.formatters.DateFormatter', parameters: function () { return [ 'String', true ]; }},
        'format': { type: 'String', declaredBy: 'mx.formatters.DateFormatter', parameters: function () { return [ 'Object', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.formatters.DateFormatter.prototype.ROYALE_COMPILE_FLAGS = 10;