/**
 * Generated by Apache Royale Compiler from mx/formatters/NumberBase.as
 * mx.formatters.NumberBase
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.formatters.NumberBase');

goog.require('mx.formatters.NumberBaseRoundType');
goog.require('org.apache.royale.utils.Language');
goog.require('XML');



/**
 *  Constructor.
 *
 *  @asparam decimalSeparatorFrom Decimal separator to use
 *  when parsing an input String.
 *
 *  @asparam thousandsSeparatorFrom Character to use
 *  as the thousands separator in the input String.
 *
 *  @asparam decimalSeparatorTo Decimal separator character to use
 *  when outputting formatted decimal numbers.
 *
 *  @asparam thousandsSeparatorTo Character to use
 *  as the thousands separator in the output String.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @param {string=} decimalSeparatorFrom
 * @param {string=} thousandsSeparatorFrom
 * @param {string=} decimalSeparatorTo
 * @param {string=} thousandsSeparatorTo
 */
mx.formatters.NumberBase = function(decimalSeparatorFrom, thousandsSeparatorFrom, decimalSeparatorTo, thousandsSeparatorTo) {
  decimalSeparatorFrom = typeof decimalSeparatorFrom !== 'undefined' ? decimalSeparatorFrom : ".";
  thousandsSeparatorFrom = typeof thousandsSeparatorFrom !== 'undefined' ? thousandsSeparatorFrom : ",";
  decimalSeparatorTo = typeof decimalSeparatorTo !== 'undefined' ? decimalSeparatorTo : ".";
  thousandsSeparatorTo = typeof thousandsSeparatorTo !== 'undefined' ? thousandsSeparatorTo : ",";
  ;
  this.decimalSeparatorFrom = decimalSeparatorFrom;
  this.thousandsSeparatorFrom = thousandsSeparatorFrom;
  this.decimalSeparatorTo = decimalSeparatorTo;
  this.thousandsSeparatorTo = thousandsSeparatorTo;
  this.isValid = true;
};


/**
 * @private
 * @type {string}
 */
mx.formatters.NumberBase.prototype.mx_formatters_NumberBase__decimalSeparatorFrom;


/**
 * @private
 * @type {string}
 */
mx.formatters.NumberBase.prototype.mx_formatters_NumberBase__decimalSeparatorTo;


/**
 * @private
 * @type {boolean}
 */
mx.formatters.NumberBase.prototype.mx_formatters_NumberBase__isValid = false;


/**
 * @private
 * @type {string}
 */
mx.formatters.NumberBase.prototype.mx_formatters_NumberBase__thousandsSeparatorFrom;


/**
 * @private
 * @type {string}
 */
mx.formatters.NumberBase.prototype.mx_formatters_NumberBase__thousandsSeparatorTo;


/**
 *  Formats a number by rounding it.
 *  The possible rounding types are defined by
 *  mx.formatters.NumberBaseRoundType.
 *
 *  @asparam value Value to be rounded.
 *
 *  @asparam roundType The type of rounding to perform:
 *  NumberBaseRoundType.NONE, NumberBaseRoundType.UP,
 *  NumberBaseRoundType.DOWN, or NumberBaseRoundType.NEAREST.
 *
 *  @asreturn Formatted number.
 *
 *  @see mx.formatters.NumberBaseRoundType
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @param {string} roundType
 * @return {string}
 */
mx.formatters.NumberBase.prototype.formatRounding = function(value, roundType) {
  var /** @type {number} */ v = Number(value);
  if (roundType != mx.formatters.NumberBaseRoundType.NONE) {
    if (roundType == mx.formatters.NumberBaseRoundType.UP) {
      v = Math.ceil(v);
    } else if (roundType == mx.formatters.NumberBaseRoundType.DOWN) {
      v = Math.floor(v);
    } else if (roundType == mx.formatters.NumberBaseRoundType.NEAREST) {
      v = Math.round(v);
    } else {
      this.isValid = false;
      return "";
    }
  }
  return v.toString();
};


/**
 *  Formats a number by rounding it and setting the decimal precision.
 *  The possible rounding types are defined by
 *  mx.formatters.NumberBaseRoundType.
 *
 *  @asparam value Value to be rounded.
 *
 *  @asparam roundType The type of rounding to perform:
 *  NumberBaseRoundType.NONE, NumberBaseRoundType.UP,
 *  NumberBaseRoundType.DOWN, or NumberBaseRoundType.NEAREST.
 *
 *  @asparam precision int of decimal places to use.
 *
 *  @asreturn Formatted number.
 *
 *  @see mx.formatters.NumberBaseRoundType
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @param {string} roundType
 * @param {number} precision
 * @return {string}
 */
mx.formatters.NumberBase.prototype.formatRoundingWithPrecision = function(value, roundType, precision) {
  var /** @type {number} */ v = Number(value);
  if (roundType == mx.formatters.NumberBaseRoundType.NONE) {
    if (precision == -1)
      return v.toString();
  } else {
    if (precision < 0)
      precision = 0;
    v = v * Math.pow(10, precision);
    v = Number(v.toString());
    if (roundType == mx.formatters.NumberBaseRoundType.UP) {
      v = Math.ceil(v);
    } else if (roundType == mx.formatters.NumberBaseRoundType.DOWN) {
      v = Math.floor(v);
    } else if (roundType == mx.formatters.NumberBaseRoundType.NEAREST) {
      v = Math.round(v);
    } else {
      this.isValid = false;
      return "";
    }
    v = v / Math.pow(10, precision);
  }
  return v.toString();
};


/**
 *  Formats a number by replacing the default decimal separator, ".",
 *  with the decimal separator specified by <code>decimalSeparatorTo</code>.
 *
 *  @asparam value The String value of the Number
 *  (formatted American style ####.##).
 *
 *  @asreturn String representation of the input where "." is replaced
 *  with the decimal formatting character.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @return {string}
 */
mx.formatters.NumberBase.prototype.formatDecimal = function(value) {
  var /** @type {Array} */ parts = value.split(".");
  return parts.join(this.decimalSeparatorTo);
};


/**
 *  Formats a number by using
 *  the <code>thousandsSeparatorTo</code> property as the thousands separator
 *  and the <code>decimalSeparatorTo</code> property as the decimal separator.
 *
 *  @asparam value Value to be formatted.
 *
 *  @asreturn Formatted number.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @return {string}
 */
mx.formatters.NumberBase.prototype.formatThousands = function(value) {
  var /** @type {number} */ v = Number(value);
  var /** @type {boolean} */ isNegative = (v < 0);
  var /** @type {string} */ numStr = Math.abs(v).toString();
  numStr.toLowerCase();
  var /** @type {number} */ e = (numStr.indexOf("e")) >> 0;
  if (e != -1)
    numStr = this.expandExponents(numStr);
  var /** @type {Array} */ numArr = numStr.split((numStr.indexOf(this.decimalSeparatorTo) != -1) ? this.decimalSeparatorTo : ".");
  var /** @type {number} */ numLen = (String(numArr[0]).length) >> 0;
  if (numLen > 3) {
    var /** @type {number} */ numSep = org.apache.royale.utils.Language._int(Math.floor(numLen / 3));
    if ((numLen % 3) == 0)
      numSep--;
    var /** @type {number} */ b = numLen;
    var /** @type {number} */ a = (b - 3) >> 0;
    var /** @type {Array} */ arr = [];
    for (var /** @type {number} */ i = 0; i <= numSep; i++) {
      arr[i] = numArr[0].slice(a, b);
      a = org.apache.royale.utils.Language._int(Math.max(a - 3, 0));
      b = org.apache.royale.utils.Language._int(Math.max(b - 3, 1));
    }
    arr.reverse();
    numArr[0] = arr.join(this.thousandsSeparatorTo);
  }
  numStr = numArr.join(this.decimalSeparatorTo);
  if (isNegative)
    numStr = "-" + numStr;
  return numStr.toString();
};


/**
 *  Formats a number by setting its decimal precision by using
 *  the <code>decimalSeparatorTo</code> property as the decimal separator.
 *
 *  @asparam value Value to be formatted.
 *
 *  @asparam precision Number of decimal points to use.
 *
 *  @asreturn Formatted number.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @param {number} precision
 * @return {string}
 */
mx.formatters.NumberBase.prototype.formatPrecision = function(value, precision) {
  if (precision == -1)
    return value;
  var /** @type {Array} */ numArr = value.split(this.decimalSeparatorTo);
  numArr[0] = numArr[0].length == 0 ? "0" : numArr[0];
  if (precision > 0) {
    var /** @type {string} */ decimalVal = numArr[1] ? String(numArr[1]) : "";
    var /** @type {string} */ fraction = decimalVal + "000000000000000000000000000000000";
    value = numArr[0] + this.decimalSeparatorTo + fraction.substr(0, precision);
  } else {
    value = String(numArr[0]);
  }
  return value.toString();
};


/**
 *  Formats a negative number with either a minus sign (-)
 *  or parentheses ().
 *
 *  @asparam value Value to be formatted.
 *
 *  @asparam useSign If <code>true</code>, use a minus sign (-).
 *  If <code>false</code>, use parentheses ().
 *
 *  @asreturn Formatted number.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @param {boolean} useSign
 * @return {string}
 */
mx.formatters.NumberBase.prototype.formatNegative = function(value, useSign) {
  if (useSign) {
    if (value.charAt(0) != "-")
      value = "-" + value;
  } else if (!useSign) {
    if (value.charAt(0) == "-")
      value = value.substr(1, value.length - 1);
    value = "(" + value + ")";
  } else {
    this.isValid = false;
    return "";
  }
  return value;
};


/**
 *  Extracts a number from a formatted String.
 *  Examines the String from left to right
 *  and returns the first number sequence.
 *  Ignores thousands separators and includes the
 *  decimal and numbers trailing the decimal.
 *
 *  @asparam str String to parse for the numeric value.
 *
 *  @asreturn Value, which can be a decimal.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} str
 * @return {string}
 */
mx.formatters.NumberBase.prototype.parseNumberString = function(str) {
  var /** @type {Array} */ splitDec = str.split(this.decimalSeparatorFrom);
  if (splitDec.length > 2)
    return null;
  var /** @type {number} */ len = (str.length) >> 0;
  var /** @type {number} */ count = 0;
  var /** @type {string} */ letter;
  var /** @type {string} */ num;
  var /** @type {boolean} */ isNegative = false;
  var /** @type {boolean} */ hasExponent = false;
  while (count < len) {
    letter = str.charAt(count);
    count++;
    if (("0" <= letter && letter <= "9") || (letter == this.decimalSeparatorFrom)) {
      var /** @type {string} */ lastLetter = str.charAt(count - 2);
      if ((lastLetter == "-") || (str.charAt(0) == "-"))
        isNegative = true;
      num = "";
      count--;
      for (var /** @type {number} */ i = count; i < len; i++) {
        letter = str.charAt(count);
        count++;
        if ("0" <= letter && letter <= "9") {
          num += letter;
        } else if (letter == this.decimalSeparatorFrom) {
          num += ".";
        } else if (letter == "e" || letter == "E") {
          num += letter;
          hasExponent = true;
        } else if (hasExponent && (letter == "-" || letter == "+")) {
          num += letter;
        } else if (letter != this.thousandsSeparatorFrom || count >= len) {
          break;
        }
      }
    }
  }
  if ((num != null) && (str != '')) {
    var /** @type {number} */ n = Number(num);
    if (n == 0)
      return '0';
  }
  if (num) {
    if (num.charAt(num.length - 1) == '.') {
      if (num.length >= 2) {
        num = num.substring(0, num.length - 1);
      } else if (num.length == 1) {
        num = '';
        isNegative = false;
      }
    }
  }
  return isNegative ? "-" + num : num;
};


/**
 *  Formats a number in exponent notation, into
 *  a number in decimal notation.
 *
 *  @asparam str String to process in exponent notation.
 *
 *  @asreturn Formatted number.
 *
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {string} value
 * @return {string}
 */
mx.formatters.NumberBase.prototype.expandExponents = function(value) {
  var /** @type {RegExp} */ regExp = /^([+-])?(\d+).?(\d*)[eE]([-+]?\d+)$/;
  var /** @type {Object} */ result = regExp.exec(value);
  var /** @type {string} */ sign = org.apache.royale.utils.Language.string(result[1]);
  var /** @type {string} */ first = org.apache.royale.utils.Language.string(result[2]);
  var /** @type {string} */ last = org.apache.royale.utils.Language.string(result[3]);
  var /** @type {number} */ exp = org.apache.royale.utils.Language._int(result[4]);
  if (!exp) {
    return (sign ? sign : "") + (first ? first : "0") + (last ? "." + last : "");
  }
  var /** @type {string} */ r = first + last;
  var /** @type {boolean} */ decimal = exp < 0;
  if (decimal) {
    var /** @type {number} */ o = -1 * (first.length + exp) + 1;
    return (sign ? sign : "") + "0." + new Array(o).join("0") + r;
  } else {
    var /** @type {number} */ i = exp + first.length;
    if (i >= r.length)
      return (sign ? sign : "") + r + new Array(i - r.length + 1).join("0");
    else
      return (sign ? sign : "") + r.substr(0, i) + "." + r.substr(i);
  }
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.formatters.NumberBase.prototype.decimalSeparatorFrom;


mx.formatters.NumberBase.prototype.get__decimalSeparatorFrom = function() {
  return this.mx_formatters_NumberBase__decimalSeparatorFrom;
};


mx.formatters.NumberBase.prototype.set__decimalSeparatorFrom = function(value) {
  this.mx_formatters_NumberBase__decimalSeparatorFrom = value;
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.formatters.NumberBase.prototype.decimalSeparatorTo;


mx.formatters.NumberBase.prototype.get__decimalSeparatorTo = function() {
  return this.mx_formatters_NumberBase__decimalSeparatorTo;
};


mx.formatters.NumberBase.prototype.set__decimalSeparatorTo = function(value) {
  this.mx_formatters_NumberBase__decimalSeparatorTo = value;
};


/**
 * @nocollapse
 * @export
 * @type {boolean}
 */
mx.formatters.NumberBase.prototype.isValid;


mx.formatters.NumberBase.prototype.get__isValid = function() {
  return this.mx_formatters_NumberBase__isValid;
};


mx.formatters.NumberBase.prototype.set__isValid = function(value) {
  this.mx_formatters_NumberBase__isValid = value;
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.formatters.NumberBase.prototype.thousandsSeparatorFrom;


mx.formatters.NumberBase.prototype.get__thousandsSeparatorFrom = function() {
  return this.mx_formatters_NumberBase__thousandsSeparatorFrom;
};


mx.formatters.NumberBase.prototype.set__thousandsSeparatorFrom = function(value) {
  this.mx_formatters_NumberBase__thousandsSeparatorFrom = value;
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.formatters.NumberBase.prototype.thousandsSeparatorTo;


mx.formatters.NumberBase.prototype.get__thousandsSeparatorTo = function() {
  return this.mx_formatters_NumberBase__thousandsSeparatorTo;
};


mx.formatters.NumberBase.prototype.set__thousandsSeparatorTo = function(value) {
  this.mx_formatters_NumberBase__thousandsSeparatorTo = value;
};


Object.defineProperties(mx.formatters.NumberBase.prototype, /** @lends {mx.formatters.NumberBase.prototype} */ {
/**
 * @type {string}
 */
decimalSeparatorFrom: {
get: mx.formatters.NumberBase.prototype.get__decimalSeparatorFrom,
set: mx.formatters.NumberBase.prototype.set__decimalSeparatorFrom},
/**
 * @type {string}
 */
decimalSeparatorTo: {
get: mx.formatters.NumberBase.prototype.get__decimalSeparatorTo,
set: mx.formatters.NumberBase.prototype.set__decimalSeparatorTo},
/**
 * @type {boolean}
 */
isValid: {
get: mx.formatters.NumberBase.prototype.get__isValid,
set: mx.formatters.NumberBase.prototype.set__isValid},
/**
 * @type {string}
 */
thousandsSeparatorFrom: {
get: mx.formatters.NumberBase.prototype.get__thousandsSeparatorFrom,
set: mx.formatters.NumberBase.prototype.set__thousandsSeparatorFrom},
/**
 * @type {string}
 */
thousandsSeparatorTo: {
get: mx.formatters.NumberBase.prototype.get__thousandsSeparatorTo,
set: mx.formatters.NumberBase.prototype.set__thousandsSeparatorTo}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.formatters.NumberBase.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'NumberBase', qName: 'mx.formatters.NumberBase', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.formatters.NumberBase.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    accessors: function () {
      return {
        'decimalSeparatorFrom': { type: 'String', access: 'readwrite', declaredBy: 'mx.formatters.NumberBase'},
        'decimalSeparatorTo': { type: 'String', access: 'readwrite', declaredBy: 'mx.formatters.NumberBase'},
        'isValid': { type: 'Boolean', access: 'readwrite', declaredBy: 'mx.formatters.NumberBase'},
        'thousandsSeparatorFrom': { type: 'String', access: 'readwrite', declaredBy: 'mx.formatters.NumberBase'},
        'thousandsSeparatorTo': { type: 'String', access: 'readwrite', declaredBy: 'mx.formatters.NumberBase'}
      };
    },
    methods: function () {
      return {
        'NumberBase': { type: '', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', true ,'String', true ,'String', true ,'String', true ]; }},
        'formatRounding': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ,'String', false ]; }},
        'formatRoundingWithPrecision': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ,'String', false ,'int', false ]; }},
        'formatDecimal': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ]; }},
        'formatThousands': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ]; }},
        'formatPrecision': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ,'int', false ]; }},
        'formatNegative': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ,'Boolean', false ]; }},
        'parseNumberString': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ]; }},
        'expandExponents': { type: 'String', declaredBy: 'mx.formatters.NumberBase', parameters: function () { return [ 'String', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.formatters.NumberBase.prototype.ROYALE_COMPILE_FLAGS = 10;