/**
 * Generated by Apache Royale Compiler from mx/formatters/StringFormatter.as
 * mx.formatters.StringFormatter
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.formatters.StringFormatter');
goog.require('XML');
goog.require('org.apache.royale.utils.Language');



/**
 *  Constructor.
 *
 *  @asparam format String that contains the desired format.
 *
 *  @asparam tokens String that contains the character tokens
 *  within the specified format String that is replaced
 *  during data formatting operations.
 *
 *  @asparam extractTokenFunc The token 
 *  accessor method to call when formatting for display.
 *  The method signature is
 *  value: anything, tokenInfo: {token: id, begin: start, end: finish}.
 *  This method must return a String representation of value
 *  for the specified <code>tokenInfo</code>.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @param {string} format
 * @param {string} tokens
 * @param {Function} extractTokenFunc
 */
mx.formatters.StringFormatter = function(format, tokens, extractTokenFunc) {
  ;
  this.mx_formatters_StringFormatter_formatPattern(format, tokens);
  this.mx_formatters_StringFormatter_extractToken = extractTokenFunc;
};


/**
 * @private
 * @type {Function}
 */
mx.formatters.StringFormatter.prototype.mx_formatters_StringFormatter_extractToken;


/**
 * @private
 * @type {string}
 */
mx.formatters.StringFormatter.prototype.mx_formatters_StringFormatter_reqFormat;


/**
 * @private
 * @type {Array}
 */
mx.formatters.StringFormatter.prototype.mx_formatters_StringFormatter_patternInfo;


/**
 *  Returns the formatted String using the format, tokens,
 *  and extraction function passed to the constructor.
 *
 *  @asparam value String to be formatted.
 *
 *  @asreturn Formatted String.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {Object} value
 * @return {string}
 */
mx.formatters.StringFormatter.prototype.formatValue = function(value) {
  var /** @type {Object} */ curTokenInfo = this.mx_formatters_StringFormatter_patternInfo[0];
  var /** @type {string} */ result = this.mx_formatters_StringFormatter_reqFormat.substring(0, Number(curTokenInfo.begin)) + this.mx_formatters_StringFormatter_extractToken(value, curTokenInfo);
  var /** @type {Object} */ lastTokenInfo = curTokenInfo;
  var /** @type {number} */ n = (this.mx_formatters_StringFormatter_patternInfo.length) >> 0;
  for (var /** @type {number} */ i = 1; i < n; i++) {
    curTokenInfo = this.mx_formatters_StringFormatter_patternInfo[i];
    result += this.mx_formatters_StringFormatter_reqFormat.substring(Number(lastTokenInfo.end), Number(curTokenInfo.begin)) + this.mx_formatters_StringFormatter_extractToken(value, curTokenInfo);
    lastTokenInfo = curTokenInfo;
  }
  if (lastTokenInfo.end > 0 && lastTokenInfo.end != this.mx_formatters_StringFormatter_reqFormat.length)
    result += this.mx_formatters_StringFormatter_reqFormat.substring(Number(lastTokenInfo.end));
  return result;
};


/**
 *  @asprivate
 *  Formats a user-defined pattern String into a more usable object.
 *
 *  @asparam format String that defines the user-requested pattern.
 *
 *  @asparam tokens List of valid patttern characters.
 * @private
 * @param {string} format
 * @param {string} tokens
 */
mx.formatters.StringFormatter.prototype.mx_formatters_StringFormatter_formatPattern = function(format, tokens) {
  var /** @type {number} */ start = 0;
  var /** @type {number} */ finish = 0;
  var /** @type {number} */ index = 0;
  var /** @type {Array} */ tokenArray = tokens.split(",");
  this.mx_formatters_StringFormatter_reqFormat = format;
  this.mx_formatters_StringFormatter_patternInfo = [];
  var /** @type {number} */ n = (tokenArray.length) >> 0;
  for (var /** @type {number} */ i = 0; i < n; i++) {
    start = (this.mx_formatters_StringFormatter_reqFormat.indexOf(org.apache.royale.utils.Language.string(tokenArray[i]))) >> 0;
    if (start >= 0 && start < this.mx_formatters_StringFormatter_reqFormat.length) {
      finish = (this.mx_formatters_StringFormatter_reqFormat.lastIndexOf(org.apache.royale.utils.Language.string(tokenArray[i]))) >> 0;
      finish = (finish >= 0 ? finish + 1 : start + 1) >> 0;
      this.mx_formatters_StringFormatter_patternInfo.splice(index, 0, {token:tokenArray[i], begin:start, end:finish});
      index++;
    }
  }
  this.mx_formatters_StringFormatter_patternInfo.sort(org.apache.royale.utils.Language.closure(this.mx_formatters_StringFormatter_compareValues, this, 'mx_formatters_StringFormatter_compareValues'));
};


/**
 *  @asprivate
 * @private
 * @param {Object} a
 * @param {Object} b
 * @return {number}
 */
mx.formatters.StringFormatter.prototype.mx_formatters_StringFormatter_compareValues = function(a, b) {
  if (a.begin > b.begin)
    return 1;
  else if (a.begin < b.begin)
    return -1;
  else
    return 0;
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.formatters.StringFormatter.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'StringFormatter', qName: 'mx.formatters.StringFormatter', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.formatters.StringFormatter.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    methods: function () {
      return {
        'StringFormatter': { type: '', declaredBy: 'mx.formatters.StringFormatter', parameters: function () { return [ 'String', false ,'String', false ,'Function', false ]; }},
        'formatValue': { type: 'String', declaredBy: 'mx.formatters.StringFormatter', parameters: function () { return [ 'Object', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.formatters.StringFormatter.prototype.ROYALE_COMPILE_FLAGS = 10;