/**
 * Generated by Apache Royale Compiler from mx/globalization/LocaleID.as
 * mx.globalization.LocaleID
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.globalization.LocaleID');
goog.require('XML');



/**
 *  Constructs a new LocaleID object, given a locale name. The locale name
 *  must conform to the syntax defined by the Unicode Technical Standard #35 
 *  (<a href="http://unicode.org/reports/tr35/">http://unicode.org/reports/tr35/</a>).
 *
 *  <p>When the constructor completes successfully the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>When the requested locale ID name is not available then the 
 *  <code>lastOperationStatus</code> is set to one of the following:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.USING_FALLBACK_WARNING</code></li>
 *  <li><code>LastOperationStatus.USING_DEFAULT_WARNING</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one 
 *  of the constants defined in the LastOperationStatus class.</p> 
 *
 *  <p>For details on the warnings listed above and other possible values of 
 *  the <code>lastOperationStatus</code> property see the descriptions in the 
 *  <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam name A locale ID name, which can also include an optional collation
 *          string.  For example: <code>"en-US"</code> or 
 *          <code>"de-DE&#64;collation=phonebook"</code>
 *
 *  @throws ArgumentError if the name is null.
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @constructor
 * @param {string} name
 */
mx.globalization.LocaleID = function(name) {
  this.name = name;
};


/**
 * @nocollapse
 * @const
 * @type {string}
 */
mx.globalization.LocaleID.DEFAULT = "i-default";


/**
 * @type {string}
 */
mx.globalization.LocaleID.prototype.lastOperationStatus;


/**
 * @type {string}
 */
mx.globalization.LocaleID.prototype.name;


/**
 *  Returns a list of acceptable locales based on a list of desired locales 
 *  and a list of the locales that are currently available.
 *
 *  <p>The resulting list is sorted according in order of preference.</p>
 *
 *  <p>Here is a typical use case for this method:</p>
 *
 *  <ul>
 *  <li>A user specifies a list of languages that she understands (stored in a
 *  user profile, a browser setting, or a cookie).  The user lists the 
 *  languages that she understands best first, so the order of the languages in
 *  the list is relevant.  This list is the "want" list.</li>
 *  <li>The application is localized into a number of different languages.  
 *  This list is the "have" list.</li>
 *  <li>The <code>determinePreferredLocales()</code> method returns an 
 *  intersection of the two lists, sorted so that the user's preferred 
 *  languages come first.</li>
 *  </ul>
 *
 *  <p>If this feature is not supported on the current operating system, this 
 *  method returns a null value.</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one 
 *  of the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam want A list of the user's preferred locales sorted in order of 
 *          preference. 
 *  @asparam have A list of locales available to the application. The order of 
 *          this list is not important. 
 *  @asparam keyword A keyword to use to help determine the best fit.
 *
 *  @asreturn A subset of the available locales, sorted according to the user's 
 *          preferences.
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @param {Array.<string>} want
 * @param {Array.<string>} have
 * @param {string=} keyword
 * @return {Array.<string>}
 */
mx.globalization.LocaleID.prototype.determinePreferredLocales = function(want, have, keyword) {
  keyword = typeof keyword !== 'undefined' ? keyword : "userinterface";
  return null;
};


/**
 *  Returns an object containing all of the key and value pairs from the 
 *  LocaleID object.
 *
 *  <p>The returned object is structured as a hash table or associative array,
 *  where each property name represents a key and the value of the property is
 *  value for that key. For example, the following code lists all of the keys 
 *  and values obtained from the LocaleID object using the 
 *  <code>getKeysAndValues()</code> method:</p>
 *
 *  <pre>
 *  var myLocale:LocaleID = new LocaleID("fr-CA");
 *  var localeData:Object = myLocale.getKeysAndValues();
 *  for (var propertyName:String in localeData)
 *  {
 *    trace(propertyName + " = " + localeData[propertyName]);
 *  }
 *  </pre>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asreturn An Object containing all the keys and values in the LocaleID 
 *          object, structured as an associative array or hashtable. 
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @return {Object}
 */
mx.globalization.LocaleID.prototype.getKeysAndValues = function() {
  return null;
};


/**
 *  Returns the language code specified by the locale ID name.
 *
 *  <p>If the locale name cannot be properly parsed then the language code is
 *  the same as the full locale name.</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asreturn A two-character language code obtained by parsing the locale ID 
 *          name.
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus	 
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @return {string}
 */
mx.globalization.LocaleID.prototype.getLanguage = function() {
  return "";
};


/**
 *  Returns the region code specified by the locale ID name.
 *
 *  <p>This method returns an empty string if the region code cannot be parsed
 *  or guessed This could occur if an unknown or incomplete locale ID name like
 *  "xy" is used.  The region code is not validated against a fixed list. For 
 *  example, the region code returned for a locale ID name of "xx-YY" is 
 *  "YY".</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>If the region is not part of the specified locale name, the most likely
 *  region code for the locale is "guessed" and 
 *  <code>lastOperationStatus</code> property is set to 
 *  <code>LastOperationStatus.USING_FALLBACK_WARNING</code></p>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to 
 *  one of the constants defined in the <code>LastOperationStatus</code> 
 *  class.</p>
 *
 *  @asreturn A two-character region code, or an empty string if the region code
 *          cannot be parsed or otherwise determined from the locale name.
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @return {string}
 */
mx.globalization.LocaleID.prototype.getRegion = function() {
  return "";
};


/**
 *  Returns the script code specified by the locale ID name.
 *
 *  <p>This method returns an empty string if the script code cannot be parsed
 *  or guessed This could occur if an unknown or incomplete locale ID name like
 *  "xy" is used.  The script code is not validated against a fixed list. For 
 *  example, the script code returned for a locale ID name of "xx-Abcd-YY" is 
 *  "Abcd".</p>
 *
 *  <p>The region, as well as the language, can also affect the return value. 
 *  For example, the script code for "mn-MN" (Mongolian-Mongolia) is "Cyrl" 
 *  (Cyrillic), while the script code for "mn-CN" (Mongolian-China) is "Mong"
 *  (Mongolian).</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>If the script code is not part of the specified locale name, the most 
 *  likely script code is "guessed" and <code>lastOperationStatus</code> 
 *  property is set to 
 *  <code>LastOperationStatus.USING_FALLBACK_WARNING</code>.</p>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asreturn A four-character script code, or an empty string if the script code
 *          cannot be parsed or otherwise determined from the locale name. 
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @return {string}
 */
mx.globalization.LocaleID.prototype.getScript = function() {
  return "";
};


/**
 *  Returns the language variant code specified by the locale ID name.
 *
 *  <p>This method returns an empty string if there is no language variant code
 *  in the given locale ID name.  (No guessing is necessary because few locales
 *  have or need a language variant.)</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asreturn A language variant code, or an empty string if the locale ID name 
 *          does not contain a language variant code. 
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @return {string}
 */
mx.globalization.LocaleID.prototype.getVariant = function() {
  return "";
};


/**
 *  Specifies whether the text direction for the specified locale is right to 
 *  left.
 *
 *  <p>The result can be used to determine the direction of the text in the 
 *  Flash text engine, and to decide whether to mirror the user interface to 
 *  support the current text direction.</p>
 *
 *  <p>When this method is called and it completes successfully, the
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asreturn <code>true</code> if the general text flows in a line of text 
 *          should go from right to left; otherwise <code>false</code>; 
 *
 *  @see flashx.textLayout.formats.Direction
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @return {boolean}
 */
mx.globalization.LocaleID.prototype.isRightToLeft = function() {
  return false;
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.globalization.LocaleID.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'LocaleID', qName: 'mx.globalization.LocaleID', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.globalization.LocaleID.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'lastOperationStatus': { type: 'String', get_set: function (/** mx.globalization.LocaleID */ inst, /** * */ v) {return v !== undefined ? inst.lastOperationStatus = v : inst.lastOperationStatus;}},
        'name': { type: 'String', get_set: function (/** mx.globalization.LocaleID */ inst, /** * */ v) {return v !== undefined ? inst.name = v : inst.name;}}
      };
    },
    methods: function () {
      return {
        'LocaleID': { type: '', declaredBy: 'mx.globalization.LocaleID', parameters: function () { return [ 'String', false ]; }},
        'determinePreferredLocales': { type: 'Vector.<String>', declaredBy: 'mx.globalization.LocaleID', parameters: function () { return [ 'Vector.<String>', false ,'Vector.<String>', false ,'String', true ]; }},
        'getKeysAndValues': { type: 'Object', declaredBy: 'mx.globalization.LocaleID'},
        'getLanguage': { type: 'String', declaredBy: 'mx.globalization.LocaleID'},
        'getRegion': { type: 'String', declaredBy: 'mx.globalization.LocaleID'},
        'getScript': { type: 'String', declaredBy: 'mx.globalization.LocaleID'},
        'getVariant': { type: 'String', declaredBy: 'mx.globalization.LocaleID'},
        'isRightToLeft': { type: 'Boolean', declaredBy: 'mx.globalization.LocaleID'}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.globalization.LocaleID.prototype.ROYALE_COMPILE_FLAGS = 10;