/**
 * Generated by Apache Royale Compiler from mx/globalization/NumberFormatter.as
 * mx.globalization.NumberFormatter
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.globalization.NumberFormatter');

goog.require('mx.formatters.NumberBase');
goog.require('mx.formatters.NumberFormatter');
goog.require('mx.globalization.LastOperationStatus');
goog.require('mx.globalization.LocaleID');
goog.require('mx.globalization.NumberParseResult');
goog.require('XML');



/**
 *  Constructs a new NumberFormatter object to format numbers according to the
 *  conventions of a given locale.
 *
 *  <p>This constructor determines if the current operating system supports the
 *  requested locale ID name.  If it is not supported then a fallback locale is
 *  used instead.  If a fallback locale is used then the the 
 *  <code>lastOperationStatus</code> property indicates the type of fallback, 
 *  and the <code>actualLocaleIDName</code> property contains the name of the 
 *  fallback locale ID. </p>
 *
 *  <p>To format based on the user's current operating system preferences, pass
 *  the value <code>LocaleID.DEFAULT</code> in the 
 *  <code>requestedLocaleIDName</code> parameter to the constructor.</p>
 *
 *  <p>When the constructor completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>When the requested locale ID name is not available then the 
 *  <code>lastOperationStatus</code> is set to one of the following:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.USING_FALLBACK_WARNING</code></li>
 *  <li><code>LastOperationStatus.USING_DEFAULT_WARNING</code></li>
 *  </ul>
 *
 *  <p>If this class is not supported on the current operating system, then 
 *  the <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.UNSUPPORTED_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the LastOperationStatus class.</p>
 *
 *  <p><b>For details on the warnings listed above and other possible values of
 *  the <code>lastOperationStatus</code> property see the descriptions in the 
 *  <code>LastOperationStatus</code> class.</b></p>
 *
 *  @asparam requestedLocaleIDName The preferred locale ID name to use when 
 *          determining number formats.
 *
 *  @throws TypeError if the <code>requestedLocaleIDName</code> is 
 *          <code>null</code>
 *
 *  @see LocaleID
 *  @see #requestedLocaleIDName
 *  @see #actualLocaleIDName
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @constructor
 * @param {string} requestedLocaleIDName
 */
mx.globalization.NumberFormatter = function(requestedLocaleIDName) {
  if (requestedLocaleIDName == mx.globalization.LocaleID.DEFAULT) {
  }
  this.actualLocaleIDName = "";
  this.lastOperationStatus = mx.globalization.LastOperationStatus.USING_DEFAULT_WARNING;
  this.requestedLocaleIDName = requestedLocaleIDName;
  this.mx_globalization_NumberFormatter_fmt = new mx.formatters.NumberFormatter();
  this.decimalSeparator = "";
  this.digitsType = 0;
  this.fractionalDigits = 0;
  this.groupingPattern = "";
  this.groupingSeparator = "";
  this.leadingZero = false;
  this.negativeNumberFormat = 0;
  this.negativeSymbol = "";
  this.trailingZeros = false;
  this.useGrouping = false;
};


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.actualLocaleIDName;


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.lastOperationStatus;


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.requestedLocaleIDName;


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.decimalSeparator;


/**
 * @type {number}
 */
mx.globalization.NumberFormatter.prototype.digitsType = 0;


/**
 * @type {number}
 */
mx.globalization.NumberFormatter.prototype.fractionalDigits = 0;


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.groupingPattern;


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.groupingSeparator;


/**
 * @type {boolean}
 */
mx.globalization.NumberFormatter.prototype.leadingZero;


/**
 * @type {number}
 */
mx.globalization.NumberFormatter.prototype.negativeNumberFormat = 0;


/**
 * @type {string}
 */
mx.globalization.NumberFormatter.prototype.negativeSymbol;


/**
 * @type {boolean}
 */
mx.globalization.NumberFormatter.prototype.trailingZeros;


/**
 * @type {boolean}
 */
mx.globalization.NumberFormatter.prototype.useGrouping;


/**
 *  Lists all of the locale ID names supported by this class. 
 *
 *  <p>If this class is not supported on the current operating system, this 
 *  method returns a null value.</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asreturn A vector of strings containing all of the locale ID names supported
 *          by this class.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @nocollapse
 * @return {Array.<string>}
 */
mx.globalization.NumberFormatter.getAvailableLocaleIDNames = function() {
  return null;
};


/**
 *  Formats an int value.
 *
 *  This function is equivalent to the <code>formatNumber()</code> method 
 *  except that it takes an <code>int</code> value.  If the value passed in is
 *  too large or small, such as a value greater than 1.72e308 or less than 
 *  1.72e-308, then this function returns 0.
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam value An int value to format.
 *
 *  @asreturn A formatted number string.
 *
 *  @throws MemoryError for any internal memory allocation problems.
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @param {number} value
 * @return {string}
 */
mx.globalization.NumberFormatter.prototype.formatInt = function(value) {
  var /** @type {string} */ s;
  s = this.mx_globalization_NumberFormatter_fmt.format(value);
  this.lastOperationStatus = mx.globalization.LastOperationStatus.NO_ERROR;
  return s;
};


/**
 *  Formats a Number value.
 *
 *  <p>This function formats the number based on the property values of the 
 *  formatter. If the properties are not modified after the the numberFormatter
 *  object is created, the numbers are formatted according to the locale 
 *  specific conventions provided by the operating system for the locale 
 *  identified by actualLocaleIDName.  To customize the format, the properties 
 *  can be altered to control specific aspects of formatting a number.</p>
 *
 *  <p> Very large numbers and very small magnitude numbers can be formatted 
 *  with this function. However, the number of significant digits is limited to
 *  the precision provided by the Number object. Scientific notation is not 
 *  supported.</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam value A Number value to format.
 *
 *  @asreturn A formatted number string.
 *
 *  @throws MemoryError if there are any internal memory allocation problems.
 *
 *  @see lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @param {number} value
 * @return {string}
 */
mx.globalization.NumberFormatter.prototype.formatNumber = function(value) {
  var /** @type {string} */ s;
  s = this.mx_globalization_NumberFormatter_fmt.format(value);
  this.lastOperationStatus = mx.globalization.LastOperationStatus.NO_ERROR;
  return s;
};


/**
 *  Formats a uint value.
 *
 *  This function is equivalent to the <code>formatNumber()</code> method 
 *  except that it takes a <code>uint</code>.  If the value passed in is too 
 *  large, such as a value greater than 1.72e308, then this function returns 0.
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam value A uint value.
 *
 *  @asreturn A formatted number string.
 *
 *  @throws MemoryError if there are any internal memory allocation problems.
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @param {number} value
 * @return {string}
 */
mx.globalization.NumberFormatter.prototype.formatUint = function(value) {
  var /** @type {string} */ s;
  s = this.mx_globalization_NumberFormatter_fmt.format(value);
  this.lastOperationStatus = mx.globalization.LastOperationStatus.NO_ERROR;
  return s;
};


/**
 *  Parses a string and returns a NumberParseResult object containing the 
 *  parsed elements. 
 *
 *  <p>The NumberParseResult object contains the value of the first number 
 *  found in the input string, the starting index for the number within the 
 *  string, and the index of the first character after the number in the 
 *  string.</p>
 *
 *  <p> If the string does not contain a number, the value property of the 
 *  NumberParseResult is set to <code>NaN</code> and the 
 *  <code>startIndex</code> and <code>endIndex</code> properties are set to the
 *  hexadecimal value <code>0x7fffffff</code>.</p>
 *
 *  <p>This function uses the value of the <code>decimalSeparator</code> 
 *  property to determine the portion of the number that contains fractional
 *  digits, and the <code>groupingSeparator</code> property to determine which 
 *  characters are allowed within the digits of a number, and the 
 *  <code>negativeNumberFormat</code> property to control how negative values 
 *  are represented. </p>
 *
 *  <p>The following table identifies the result of strings parsed for the 
 *  various NegativeNumberFormat values:</p>
 *
 *  <table class="innertable">
 *  <tbody>
 *  <tr>
 *  <td>NegativeNumberFormat</td>
 *  <td>Input String</td>
 *  <td>Result</td>
 *  </tr>
 *  <tr>
 *  <td>(n)</td>
 *  <td>"(123)" or "( 123 )"</td>
 *  <td>"-123"</td>
 *  </tr>
 *  <tr>
 *  <td>-n</td>
 *  <td>"-123" or "- 123"</td>
 *  <td>"-123"</td>
 *  </tr>
 *  <tr>
 *  <td>- n</td>
 *  <td>"-123" or "- 123"</td>
 *  <td>"-123"</td>
 *  </tr>
 *  <tr>
 *  <td>n-</td>
 *  <td>"123-" or "123 -"</td>
 *  <td>"-123"</td>
 *  </tr>
 *  <tr>
 *  <td>n -</td>
 *  <td>"123-" or "123 -"</td>
 *  <td>"-123"</td>
 *  </tr>
 *  </tbody>
 *  </table>
 *
 *  <p>A single white space is allowed between the number and the minus sign or
 *  parenthesis.</p>
 *
 *  <p>Other properties are ignored when determining a valid number. 
 *  Specifically the value of the <code>digitsType</code> property is ignored 
 *  and the digits can be from any of the digit sets that are enumerated in 
 *  the NationalDigitsType class. The values of the 
 *  <code>groupingPattern</code> and <code>useGrouping</code> properties do 
 *  not influence the parsing of the number.</p>
 *
 *  <p> If numbers are preceded or followed in the string by a plus sign '+', 
 *  the plus sign is treated as a character that is not part of the number.</p>
 *
 *  <p> This function does not parse strings containing numbers in scientific 
 *  notation (e.g. 1.23e40).</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam parseString
 *
 *  @asreturn 
 *
 *  @throws TypeError if the parseString is <code>null</code>
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *  @see NumberParseResult
 *  @see #parseNumber()
 *  @see #parseFloat()
 *  @see NationalDigitsType
 *
 *  @example The following code parses a number from a string and retrieves the
 *          prefix and suffix: 
 *          <listing version="3.0">
 *
 *          var nf:NumberFormatter = new NumberFormatter("fr-FR"); 
 *          var str:String = "1,56 mètre"
 *          var result:NumberParseResult = nf.parse(str);
 *          trace(result.value) // 1.56
 *          trace(str.substr(0,result.startIndex));                // ""
 *          trace(str.substr(result.startIndex, result.endIndex)); // "1,56"
 *          trace(str.substr(result.endIndex));                    // " mètre"
 *          </listing>
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @param {string} parseString
 * @return {mx.globalization.NumberParseResult}
 */
mx.globalization.NumberFormatter.prototype.parse = function(parseString) {
  return new mx.globalization.NumberParseResult(this.parseNumber(parseString));
};


/**
 *  Parses a string that contains only digits and optional whitespace 
 *  characters and returns a Number. 
 *
 *  If the string does not begin with a number or contains characters other 
 *  than whitespace that are not part of the number, then this method returns 
 *  <code>NaN</code>. White space before or after the numeric digits is 
 *  ignored. A white space character is a character that has a Space Separator 
 *  (Zs) property in the Unicode Character Database 
 *  (see http://www.unicode.org/ucd/). 
 *
 *  <p> If the numeric digit is preceded or followed by a plus sign '+' it is 
 *  treated as a non-whitespace character.  The return value is 
 *  <code>NaN</code>.</p>
 *
 *  <p> See the description of the parse function for more information about 
 *  number parsing and what constitutes a valid number.</p>
 *
 *  <p>When this method is called and it completes successfully, the 
 *  <code>lastOperationStatus</code> property is set to:</p>
 *
 *  <ul>
 *  <li><code>LastOperationStatus.NO_ERROR</code></li>
 *  </ul>
 *
 *  <p>Otherwise the <code>lastOperationStatus</code> property is set to one of
 *  the constants defined in the <code>LastOperationStatus</code> class.</p>
 *
 *  @asparam parseString
 *
 *  @asreturn
 *
 *  @throws TypeError if the parseString is <code>null</code>
 *
 *  @see #lastOperationStatus
 *  @see LastOperationStatus
 *  @see #parse()
 *  @see #parseFloat()
 *  @see NationalDigitsType
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.1
 *  @playerversion AIR 2.0
 *  @productversion Royale 0.9.8
 * @param {string} parseString
 * @return {number}
 */
mx.globalization.NumberFormatter.prototype.parseNumber = function(parseString) {
  var /** @type {mx.formatters.NumberBase} */ parser = new mx.formatters.NumberBase(".", ",", ".", ",");
  var /** @type {number} */ num = Number(parser.parseNumberString(parseString));
  return num;
};


/**
 * @private
 * @type {mx.formatters.NumberFormatter}
 */
mx.globalization.NumberFormatter.prototype.mx_globalization_NumberFormatter_fmt;


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.globalization.NumberFormatter.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'NumberFormatter', qName: 'mx.globalization.NumberFormatter', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.globalization.NumberFormatter.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'actualLocaleIDName': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.actualLocaleIDName = v : inst.actualLocaleIDName;}},
        'lastOperationStatus': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.lastOperationStatus = v : inst.lastOperationStatus;}},
        'requestedLocaleIDName': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.requestedLocaleIDName = v : inst.requestedLocaleIDName;}},
        'decimalSeparator': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.decimalSeparator = v : inst.decimalSeparator;}},
        'digitsType': { type: 'uint', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.digitsType = v : inst.digitsType;}},
        'fractionalDigits': { type: 'int', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.fractionalDigits = v : inst.fractionalDigits;}},
        'groupingPattern': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.groupingPattern = v : inst.groupingPattern;}},
        'groupingSeparator': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.groupingSeparator = v : inst.groupingSeparator;}},
        'leadingZero': { type: 'Boolean', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.leadingZero = v : inst.leadingZero;}},
        'negativeNumberFormat': { type: 'uint', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.negativeNumberFormat = v : inst.negativeNumberFormat;}},
        'negativeSymbol': { type: 'String', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.negativeSymbol = v : inst.negativeSymbol;}},
        'trailingZeros': { type: 'Boolean', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.trailingZeros = v : inst.trailingZeros;}},
        'useGrouping': { type: 'Boolean', get_set: function (/** mx.globalization.NumberFormatter */ inst, /** * */ v) {return v !== undefined ? inst.useGrouping = v : inst.useGrouping;}}
      };
    },
    methods: function () {
      return {
        'NumberFormatter': { type: '', declaredBy: 'mx.globalization.NumberFormatter', parameters: function () { return [ 'String', false ]; }},
        '|getAvailableLocaleIDNames': { type: 'Vector.<String>', declaredBy: 'mx.globalization.NumberFormatter'},
        'formatInt': { type: 'String', declaredBy: 'mx.globalization.NumberFormatter', parameters: function () { return [ 'int', false ]; }},
        'formatNumber': { type: 'String', declaredBy: 'mx.globalization.NumberFormatter', parameters: function () { return [ 'Number', false ]; }},
        'formatUint': { type: 'String', declaredBy: 'mx.globalization.NumberFormatter', parameters: function () { return [ 'uint', false ]; }},
        'parse': { type: 'mx.globalization.NumberParseResult', declaredBy: 'mx.globalization.NumberFormatter', parameters: function () { return [ 'String', false ]; }},
        'parseNumber': { type: 'Number', declaredBy: 'mx.globalization.NumberFormatter', parameters: function () { return [ 'String', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.globalization.NumberFormatter.prototype.ROYALE_COMPILE_FLAGS = 10;