/**
 * Generated by Apache Royale Compiler from mx/logging/targets/LineFormattedTarget.as
 * mx.logging.targets.LineFormattedTarget
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.logging.targets.LineFormattedTarget');

goog.require('mx.logging.AbstractTarget');
goog.require('mx.logging.ILogger');
goog.require('mx.logging.LogEvent');
goog.require('org.apache.royale.utils.Language');
goog.require('XML');



/**
 *  Constructor.
 *
 *  <p>Constructs an instance of a logger target that will format
 *  the message data on a single line.</p>
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @extends {mx.logging.AbstractTarget}
 */
mx.logging.targets.LineFormattedTarget = function() {
  mx.logging.targets.LineFormattedTarget.base(this, 'constructor');
  this.includeTime = false;
  this.includeDate = false;
  this.includeCategory = false;
  this.includeLevel = false;
};
goog.inherits(mx.logging.targets.LineFormattedTarget, mx.logging.AbstractTarget);


/**
 * @type {string}
 */
mx.logging.targets.LineFormattedTarget.prototype.fieldSeparator = " ";


/**
 * @type {boolean}
 */
mx.logging.targets.LineFormattedTarget.prototype.includeCategory;


/**
 * @type {boolean}
 */
mx.logging.targets.LineFormattedTarget.prototype.includeDate;


/**
 * @type {boolean}
 */
mx.logging.targets.LineFormattedTarget.prototype.includeLevel;


/**
 * @type {boolean}
 */
mx.logging.targets.LineFormattedTarget.prototype.includeTime;


/**
 *  This method handles a <code>LogEvent</code> from an associated logger.
 *  A target uses this method to translate the event into the appropriate
 *  format for transmission, storage, or display.
 *  This method is called only if the event's level is in range of the
 *  target's level.
 * 
 *  @asparam event The <code>LogEvent</code> handled by this method.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @override
 */
mx.logging.targets.LineFormattedTarget.prototype.logEvent = function(event) {
  var /** @type {string} */ date = "";
  if (this.includeDate || this.includeTime) {
    var /** @type {Date} */ d = new Date();
    if (this.includeDate) {
      date = Number(d.getMonth() + 1).toString() + "/" + d.getDate().toString() + "/" + d.getFullYear() + this.fieldSeparator;
    }
    if (this.includeTime) {
      date += this.mx_logging_targets_LineFormattedTarget_padTime(d.getHours()) + ":" + this.mx_logging_targets_LineFormattedTarget_padTime(d.getMinutes()) + ":" + this.mx_logging_targets_LineFormattedTarget_padTime(d.getSeconds()) + "." + this.mx_logging_targets_LineFormattedTarget_padTime(d.getMilliseconds(), true) + this.fieldSeparator;
    }
  }
  var /** @type {string} */ level = "";
  if (this.includeLevel) {
    level = "[" + mx.logging.LogEvent.getLevelString((event.level) >>> 0) + "]" + this.fieldSeparator;
  }
  var /** @type {string} */ category = this.includeCategory ? org.apache.royale.utils.Language.as(event.target, mx.logging.ILogger, true).category + this.fieldSeparator : "";
  this.internalLog(date + level + category + event.message);
};


/**
 *  @asprivate
 * @private
 * @param {number} num
 * @param {boolean=} millis
 * @return {string}
 */
mx.logging.targets.LineFormattedTarget.prototype.mx_logging_targets_LineFormattedTarget_padTime = function(num, millis) {
  millis = typeof millis !== 'undefined' ? millis : false;
  if (millis) {
    if (num < 10)
      return "00" + num.toString();
    else if (num < 100)
      return "0" + num.toString();
    else
      return num.toString();
  } else {
    return num > 9 ? num.toString() : "0" + num.toString();
  }
};


/**
 *  Descendants of this class should override this method to direct the 
 *  specified message to the desired output.
 *
 *  @asparam  message String containing preprocessed log message which may
 *              include time, date, category, etc. based on property settings,
 *              such as <code>includeDate</code>, <code>includeCategory</code>,
 *          etc.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @protected
 * @param {string} message
 */
mx.logging.targets.LineFormattedTarget.prototype.internalLog = function(message) {
};


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.logging.targets.LineFormattedTarget.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'LineFormattedTarget', qName: 'mx.logging.targets.LineFormattedTarget', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.logging.targets.LineFormattedTarget.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'fieldSeparator': { type: 'String', get_set: function (/** mx.logging.targets.LineFormattedTarget */ inst, /** * */ v) {return v !== undefined ? inst.fieldSeparator = v : inst.fieldSeparator;}},
        'includeCategory': { type: 'Boolean', get_set: function (/** mx.logging.targets.LineFormattedTarget */ inst, /** * */ v) {return v !== undefined ? inst.includeCategory = v : inst.includeCategory;}},
        'includeDate': { type: 'Boolean', get_set: function (/** mx.logging.targets.LineFormattedTarget */ inst, /** * */ v) {return v !== undefined ? inst.includeDate = v : inst.includeDate;}},
        'includeLevel': { type: 'Boolean', get_set: function (/** mx.logging.targets.LineFormattedTarget */ inst, /** * */ v) {return v !== undefined ? inst.includeLevel = v : inst.includeLevel;}},
        'includeTime': { type: 'Boolean', get_set: function (/** mx.logging.targets.LineFormattedTarget */ inst, /** * */ v) {return v !== undefined ? inst.includeTime = v : inst.includeTime;}}
      };
    },
    methods: function () {
      return {
        'LineFormattedTarget': { type: '', declaredBy: 'mx.logging.targets.LineFormattedTarget'},
        'logEvent': { type: 'void', declaredBy: 'mx.logging.targets.LineFormattedTarget', parameters: function () { return [ 'mx.logging.LogEvent', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.logging.targets.LineFormattedTarget.prototype.ROYALE_COMPILE_FLAGS = 10;