/**
 * Generated by Apache Royale Compiler from mx/messaging/channels/amfx/AMFXDecoder.as
 * mx.messaging.channels.amfx.AMFXDecoder
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.messaging.channels.amfx.AMFXDecoder');
/* Royale Static Dependency List: org.apache.royale.utils.Language*/

goog.require('XML');
goog.require('XMLList');
goog.require('mx.logging.Log');
goog.require('mx.messaging.channels.amfx.AMFXContext');
goog.require('mx.messaging.channels.amfx.AMFXHeader');
goog.require('mx.messaging.channels.amfx.AMFXResult');
goog.require('mx.messaging.errors.ChannelError');
goog.require('mx.resources.IResourceManager');
goog.require('mx.resources.ResourceManager');
goog.require('mx.utils.ByteArray');
goog.require('mx.utils.HexDecoder');
goog.require('org.apache.royale.reflection.getClassByAlias');
goog.require('org.apache.royale.reflection.getDefinitionByName');
goog.require('org.apache.royale.utils.net.IExternalizable');
goog.require('org.apache.royale.utils.Language');



/**
 * @constructor
 */
mx.messaging.channels.amfx.AMFXDecoder = function() {
  ;
};


/**
 * @private
 * @type {mx.resources.IResourceManager}
 */
mx.messaging.channels.amfx.AMFXDecoder._resourceManager;


/**
 * @param {XML} xml
 * @return {mx.messaging.channels.amfx.AMFXResult}
 */
mx.messaging.channels.amfx.AMFXDecoder.prototype.decode = function(xml) {
  if (xml) {
    var /** @type {mx.messaging.channels.amfx.AMFXContext} */ context = new mx.messaging.channels.amfx.AMFXContext();
    context.log = mx.logging.Log.getLogger("mx.messaging.channels.amfx.AMFXDecoder");
    XML.ignoreWhitespace = false;
    return mx.messaging.channels.amfx.AMFXDecoder.decodePacket(xml, context);
  } else {
    return null;
  }
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {mx.messaging.channels.amfx.AMFXResult}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodePacket = function(xml, context) {
  var /** @type {mx.messaging.channels.amfx.AMFXResult} */ result = new mx.messaging.channels.amfx.AMFXResult();
  var /** @type {string} */ message;
  var /** @type {string} */ name = org.apache.royale.utils.Language.as(xml.localName(), String);
  if (name == "amfx") {
    var /** @type {string} */ v = xml.attribute("ver").toString();
    var /** @type {number} */ version = org.apache.royale.utils.Language.uint(v);
    if (mx.messaging.channels.amfx.AMFXDecoder.supportedVersion(version)) {
      var /** @type {XMLList} */ h = xml.child("header");
      if (h) {
        result.headers = mx.messaging.channels.amfx.AMFXDecoder.decodeHeaders(h, context);
      }
      var /** @type {XML} */ body = xml.child('body')[0];
      if (body) {
        result.result = mx.messaging.channels.amfx.AMFXDecoder.decodeBody(body, context);
      } else {
        message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "noAMFXBody");
        throw new mx.messaging.errors.ChannelError(message);
      }
    } else {
      message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "unsupportedAMFXVersion", [version]);
      throw new mx.messaging.errors.ChannelError(message);
    }
  } else {
    message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "noAMFXNode");
    throw new mx.messaging.errors.ChannelError(message);
  }
  return result;
};


/**
 * @private
 * @param {XMLList} xmlList
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Array}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeHeaders = function(xmlList, context) {
  var /** @type {Array} */ headers = [];
  for (var /** @type {number} */ i = 0; i < xmlList.length(); i++) {
    var /** @type {XML} */ h = xmlList[i];
    var /** @type {string} */ name = h.attribute("name").toString();
    var /** @type {mx.messaging.channels.amfx.AMFXHeader} */ header = new mx.messaging.channels.amfx.AMFXHeader();
    header.name = name;
    var /** @type {string} */ temp = h.attribute("mustUnderstand").toString();
    header.mustUnderstand = (temp == "true");
    var /** @type {XMLList} */ children = h.children();
    if (children.length() > 0) {
      header.content = mx.messaging.channels.amfx.AMFXDecoder.decodeValue(children[0], context);
    } else {
      header.content = null;
    }
    headers[i] = header;
  }
  return headers;
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Object}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeBody = function(xml, context) {
  var /** @type {Object} */ result;
  var /** @type {XMLList} */ children = xml.children();
  if (children.length() == 1) {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeValue(children[0], context);
  } else if (children.length() > 1) {
    result = [];
    for (var /** @type {number} */ i = 0; i < children.length(); i++) {
      result[i] = mx.messaging.channels.amfx.AMFXDecoder.decodeValue(children[i], context);
    }
  }
  return result;
};


/**
 * @nocollapse
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Object}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeValue = function(xml, context) {
  var /** @type {Object} */ result;
  var /** @type {string} */ name = org.apache.royale.utils.Language.as(xml.localName(), String);
  if (name == "string") {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeString(xml, context);
  } else if (name == "true") {
    result = true;
  } else if (name == "false") {
    result = false;
  } else if (name == "array") {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeArray(xml, context);
  } else if (name == "object") {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeObject(xml, context);
  } else if (name == "ref") {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeRef(xml, context);
  } else if (name == "double") {
    var /** @type {string} */ n = xml.text().toString();
    result = Number(n);
  } else if (name == "int") {
    var /** @type {string} */ i = xml.text().toString();
    result = org.apache.royale.utils.Language._int(i);
  } else if (name == "null") {
    result = null;
  } else if (name == "date") {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeDate(xml, context);
  } else if (name == "xml") {
    var /** @type {string} */ x = xml.text().toString();
    if (mx.messaging.channels.amfx.AMFXDecoder.hasEscapedCloseCDATA(xml)) {
      x = x.replace(mx.messaging.channels.amfx.AMFXDecoder.REGEX_CLOSE_CDATA, "]]>");
    }
    result = new XML(x);
  } else if (name == "bytearray") {
    result = mx.messaging.channels.amfx.AMFXDecoder.decodeByteArray(xml);
  } else if (name == "undefined") {
    result = undefined;
  }
  return result;
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Array}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeArray = function(xml, context) {
  var /** @type {Array} */ array = [];
  context.addObject(array);
  var /** @type {XMLList} */ entries = xml.child('*');
  if (entries) {
    for (var /** @type {number} */ i = 0; i < entries.length(); i++) {
      var /** @type {XML} */ x = entries[i];
      var /** @type {Object} */ prop;
      if (x.localName() == "item") {
        var /** @type {string} */ propName = x.attribute("name").toString();
        prop = mx.messaging.channels.amfx.AMFXDecoder.decodeValue(x.child('*')[0], context);
        array[propName] = prop;
      } else {
        prop = mx.messaging.channels.amfx.AMFXDecoder.decodeValue(x, context);
        array.push(prop);
      }
    }
  }
  return array;
};


/**
 * @private
 * @param {XML} xml
 * @return {mx.utils.ByteArray}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeByteArray = function(xml) {
  var /** @type {string} */ str = xml.text().toString();
  var /** @type {mx.utils.HexDecoder} */ decoder = new mx.utils.HexDecoder();
  decoder.decode(str);
  return decoder.drain();
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Date}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeDate = function(xml, context) {
  var /** @type {string} */ d = xml.text().toString();
  var /** @type {number} */ time = Number(d);
  var /** @type {Date} */ result = new Date();
  result.setTime(time);
  context.addObject(result);
  return result;
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Object}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeObject = function(xml, context) {
  var /** @type {Object} */ result;
  var /** @type {string} */ className = xml.attribute("type").toString();
  if (className) {
    try {
      var /** @type {Object} */ classType = null;
      try {
        classType = org.apache.royale.reflection.getClassByAlias(className);
      } catch (e1) {
        if (!classType)
          classType = org.apache.royale.utils.Language.as(org.apache.royale.reflection.getDefinitionByName(className), org.apache.royale.utils.Language.synthType('Class'));
      }
      result = new classType();
    } catch (e) {
      if (context.log)
        context.log.warn("Error instantiating class: {0}. Reverting to anonymous Object.", className);
      result = {};
    }
  } else {
    className = "Object";
    result = {};
  }
  context.addObject(result);
  var /** @type {XMLList} */ entries = xml.child('*');
  if (entries && entries.length() > 0) {
    var /** @type {Object} */ traits;
    var /** @type {XML} */ tx = entries[0];
    var /** @type {string} */ message;
    if (tx.localName() == "traits") {
      traits = mx.messaging.channels.amfx.AMFXDecoder.decodeTraits(tx, className, context);
      entries.removeChild(0);
    }
    if (!traits) {
      message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "AMFXTraitsNotFirst");
      throw new mx.messaging.errors.ChannelError(message);
    }
    if (traits.externalizable) {
      if (org.apache.royale.utils.Language.is(result, org.apache.royale.utils.net.IExternalizable)) {
        var /** @type {org.apache.royale.utils.net.IExternalizable} */ ext = org.apache.royale.utils.Language.as(result, org.apache.royale.utils.net.IExternalizable, true);
        tx = entries[0];
        if (tx.localName() == "bytearray") {
          var /** @type {mx.utils.ByteArray} */ ba = mx.messaging.channels.amfx.AMFXDecoder.decodeByteArray(tx);
          try {
            ext.readExternal(ba);
          } catch (e) {
            message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "errorReadingIExternalizable", [(e.message)]);
            throw new mx.messaging.errors.ChannelError(message);
          }
        }
      } else {
        message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "notImplementingIExternalizable", [className]);
        throw new mx.messaging.errors.ChannelError(message);
      }
    } else {
      for (var /** @type {number} */ i = 0; i < entries.length(); i++) {
        var /** @type {string} */ propName = org.apache.royale.utils.Language.string(traits.properties[i]);
        var /** @type {*} */ propValue = mx.messaging.channels.amfx.AMFXDecoder.decodeValue(entries[i], context);
        try {
          result[propName] = propValue;
        } catch (e2) {
          if (context.log != null)
            context.log.warn("Cannot set property '{0}' on type '{1}'.", propName, className);
        }
      }
    }
  }
  return result;
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Object}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeRef = function(xml, context) {
  var /** @type {*} */ result;
  var /** @type {string} */ message;
  var /** @type {string} */ idAttr = xml.attribute("id").toString();
  if (idAttr) {
    var /** @type {number} */ ref = (parseInt(idAttr, 0)) >> 0;
    if (!isNaN(ref)) {
      result = context.getObject((ref) >>> 0);
    }
    if (result === undefined) {
      message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "unknownReference", [idAttr]);
      throw new mx.messaging.errors.ChannelError(message);
    }
  } else {
    message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "referenceMissingId");
    throw new mx.messaging.errors.ChannelError(message);
  }
  return result;
};


/**
 * @private
 * @param {XML} xml
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @param {boolean=} isTrait
 * @return {string}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeString = function(xml, context, isTrait) {
  isTrait = typeof isTrait !== 'undefined' ? isTrait : false;
  var /** @type {string} */ str;
  var /** @type {string} */ refAttr = xml.attribute("id").toString();
  if (refAttr) {
    var /** @type {number} */ ref = org.apache.royale.utils.Language.uint(refAttr);
    if (!isNaN(ref)) {
      str = context.getString(ref);
    }
    if (!str) {
      var /** @type {string} */ message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "unknownStringReference", [refAttr]);
      throw new mx.messaging.errors.ChannelError(message);
    }
  } else {
    str = xml.text().toString();
    if (!isTrait && mx.messaging.channels.amfx.AMFXDecoder.hasEscapedCloseCDATA(xml)) {
      str = str.replace(mx.messaging.channels.amfx.AMFXDecoder.REGEX_CLOSE_CDATA, "]]>");
    }
    context.addString(str);
  }
  return str;
};


/**
 * @private
 * @param {XML} xml
 * @param {string} className
 * @param {mx.messaging.channels.amfx.AMFXContext} context
 * @return {Object}
 */
mx.messaging.channels.amfx.AMFXDecoder.decodeTraits = function(xml, className, context) {
  var /** @type {Object} */ traits;
  var /** @type {string} */ refAttr = xml.attribute("id").toString();
  if (refAttr) {
    var /** @type {number} */ ref = org.apache.royale.utils.Language.uint(refAttr);
    if (!isNaN(ref)) {
      traits = context.getTraitInfo(ref);
    }
    if (!traits) {
      var /** @type {string} */ message = mx.messaging.channels.amfx.AMFXDecoder.resourceManager.getString("messaging", "unknownTraitReference", [refAttr]);
      throw new mx.messaging.errors.ChannelError(message);
    }
  } else {
    traits = {};
    traits.name = className;
    traits.alias = className;
    traits.properties = [];
    traits.externalizable = false;
    var /** @type {string} */ ext = xml.attribute("externalizable").toString();
    if (ext == "true") {
      traits.externalizable = true;
    }
    var /** @type {XMLList} */ nodes = xml.child('*');
    if (nodes) {
      for (var /** @type {number} */ i = 0; i < nodes.length(); i++) {
        traits.properties[i] = mx.messaging.channels.amfx.AMFXDecoder.decodeString(nodes[i], context, true);
      }
    }
    context.addTraitInfo(traits);
  }
  return traits;
};


/**
 * @private
 * @param {XML} xml
 * @return {boolean}
 */
mx.messaging.channels.amfx.AMFXDecoder.hasEscapedCloseCDATA = function(xml) {
  var /** @type {string} */ s = xml.toXMLString();
  if (s.indexOf("]]>") != -1) {
    return s.indexOf("]]&gt;") != -1;
  } else {
    return false;
  }
};


/**
 * @private
 * @param {number} ver
 * @return {boolean}
 */
mx.messaging.channels.amfx.AMFXDecoder.supportedVersion = function(ver) {
  for (var /** @type {number} */ i = 0; i < mx.messaging.channels.amfx.AMFXDecoder.SUPPORTED_VERSIONS.length; i++) {
    if (ver == mx.messaging.channels.amfx.AMFXDecoder.SUPPORTED_VERSIONS[i])
      return true;
  }
  return false;
};


/**
 * @private
 * @const
 * @type {Array}
 */
mx.messaging.channels.amfx.AMFXDecoder.SUPPORTED_VERSIONS = [3];


/**
 * @private
 * @const
 * @type {RegExp}
 */
mx.messaging.channels.amfx.AMFXDecoder.REGEX_CLOSE_CDATA = new RegExp("]]&gt;", "g");


/**
 * @nocollapse
 * @type {mx.resources.IResourceManager}
 */
mx.messaging.channels.amfx.AMFXDecoder.resourceManager;


mx.messaging.channels.amfx.AMFXDecoder.get__resourceManager = function() {
  if (!mx.messaging.channels.amfx.AMFXDecoder._resourceManager)
    mx.messaging.channels.amfx.AMFXDecoder._resourceManager = mx.resources.ResourceManager.getInstance();
  return mx.messaging.channels.amfx.AMFXDecoder._resourceManager;
};


Object.defineProperties(mx.messaging.channels.amfx.AMFXDecoder, /** @lends {mx.messaging.channels.amfx.AMFXDecoder} */ {
/**
 * @type {mx.resources.IResourceManager}
 */
resourceManager: {
get: mx.messaging.channels.amfx.AMFXDecoder.get__resourceManager}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.messaging.channels.amfx.AMFXDecoder.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'AMFXDecoder', qName: 'mx.messaging.channels.amfx.AMFXDecoder', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.messaging.channels.amfx.AMFXDecoder.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    methods: function () {
      return {
        'AMFXDecoder': { type: '', declaredBy: 'mx.messaging.channels.amfx.AMFXDecoder'},
        'decode': { type: 'mx.messaging.channels.amfx.AMFXResult', declaredBy: 'mx.messaging.channels.amfx.AMFXDecoder', parameters: function () { return [ 'XML', false ]; }},
        '|decodeValue': { type: 'Object', declaredBy: 'mx.messaging.channels.amfx.AMFXDecoder', parameters: function () { return [ 'XML', false ,'mx.messaging.channels.amfx.AMFXContext', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.messaging.channels.amfx.AMFXDecoder.prototype.ROYALE_COMPILE_FLAGS = 10;