/**
 * Generated by Apache Royale Compiler from mx/utils/ObjectProxy.as
 * mx.utils.ObjectProxy
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.utils.ObjectProxy');

goog.require('QName');
goog.require('mx.core.IPropertyChangeNotifier');
goog.require('mx.events.PropertyChangeEvent');
goog.require('mx.events.PropertyChangeEventKind');
goog.require('mx.utils.ObjectUtil');
goog.require('mx.utils.UIDUtil');
goog.require('org.apache.royale.events.Event');
goog.require('org.apache.royale.events.EventDispatcher');
goog.require('org.apache.royale.reflection.getQualifiedClassName');
goog.require('org.apache.royale.utils.Proxy');
goog.require('org.apache.royale.utils.net.IDataInput');
goog.require('org.apache.royale.utils.net.IDataOutput');
goog.require('org.apache.royale.utils.net.IExternalizable');
goog.require('org.apache.royale.utils.Language');
goog.require('XML');



/**
 *  Initializes this proxy with the specified object, id and proxy depth.
 * 
 *  @asparam item Object to proxy.
 *  If no item is specified, an anonymous object will be constructed
 *  and assigned.
 *
 *  @asparam uid String containing the unique id
 *  for this object instance.
 *  Required for IPropertyChangeNotifier compliance as every object must 
 *  provide a unique way of identifying it.
 *  If no value is specified, a random id will be assigned.
 *
 *  @asparam proxyDepth An integer indicating how many levels in a complex
 *  object graph should have a proxy created during property access.
 *  The default is -1, meaning "proxy to infinite depth".
 *  
 *  @example
 *
 *  <pre>
 *  import mx.events.PropertyChangeEvent;
 *  import mx.utils.ObjectUtil;
 *  import mx.utils.ObjectProxy;
 *  import mx.utils.StringUtil;
 *
 *  var a:Object = { name: "Tyler", age: 5, ssnum: "555-55-5555" };
 *  var p:ObjectProxy = new ObjectProxy(a);
 *  p.addEventListener(PropertyChangeEvent.PROPERTY_CHANGE, updateHandler);
 *  p.name = "Jacey";
 *  p.age = 2;
 *  delete p.ssnum;
 *
 *  // handler function
 *  function updateHandler(event:PropertyChangeEvent):void
 *  {
 *      trace(StringUtil.substitute("updateHandler('{0}', {1}, {2}, {3}, '{4}')",
 *                                     event.kind,
 *                                     event.property,
 *                                     event.oldValue,
 *                                     event.newValue,
 *                                     event.target.uid));
 *  }
 *
 *  // trace output
 *  updateHandler('opUpdate', name, Jacey, '698AF8CB-B3D9-21A3-1AFFDGHT89075CD2')
 *  updateHandler('opUpdate', age, 2, '698AF8CB-B3D9-21A3-1AFFDGHT89075CD2')
 *  updateHandler('opDelete', ssnum, null, '698AF8CB-B3D9-21A3-1AFFDGHT89075CD2')
 *  </pre>
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @extends {org.apache.royale.utils.Proxy}
 * @implements {org.apache.royale.utils.net.IExternalizable}
 * @implements {mx.core.IPropertyChangeNotifier}
 * @param {Object=} item
 * @param {string=} uid
 * @param {number=} proxyDepth
 */
mx.utils.ObjectProxy = function(item, uid, proxyDepth) {
  item = typeof item !== 'undefined' ? item : null;
  uid = typeof uid !== 'undefined' ? uid : null;
  proxyDepth = typeof proxyDepth !== 'undefined' ? proxyDepth : -1;
  
  this.proxyClass = mx.utils.ObjectProxy;
  mx.utils.ObjectProxy.base(this, 'constructor');
  if (!item)
    item = {};
  this.mx_utils_ObjectProxy__item = item;
  this.mx_utils_ObjectProxy__proxyLevel = proxyDepth;
  this.notifiers = {};
  this.dispatcher = new org.apache.royale.events.EventDispatcher(this);
  if (uid)
    this.mx_utils_ObjectProxy__id = uid;
};
goog.inherits(mx.utils.ObjectProxy, org.apache.royale.utils.Proxy);


/**
 * @protected
 * @type {org.apache.royale.events.EventDispatcher}
 */
mx.utils.ObjectProxy.prototype.dispatcher;


/**
 * @protected
 * @type {Object}
 */
mx.utils.ObjectProxy.prototype.notifiers;


/**
 * @protected
 * @type {Object}
 */
mx.utils.ObjectProxy.prototype.proxyClass;


/**
 * @protected
 * @type {Array}
 */
mx.utils.ObjectProxy.prototype.propertyList;


/**
 * @private
 * @type {number}
 */
mx.utils.ObjectProxy.prototype.mx_utils_ObjectProxy__proxyLevel = 0;


/**
 * @private
 * @type {Object}
 */
mx.utils.ObjectProxy.prototype.mx_utils_ObjectProxy__item;


/**
 * @private
 * @type {string}
 */
mx.utils.ObjectProxy.prototype.mx_utils_ObjectProxy__id;


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.getProperty = function(name) {
  var /** @type {*} */ result;
  if (org.apache.royale.utils.Language.is(name, String) && name.length > 1 && name.charAt(0) == '"' && name.charAt(name.length - 1) == '"') {
    name = name.slice(1, -1);
  }
  if (this.notifiers[name.toString()])
    return this.notifiers[name];
  result = this.mx_utils_ObjectProxy__item[name];
  if (result) {
    if (this.mx_utils_ObjectProxy__proxyLevel == 0 || mx.utils.ObjectUtil.isSimple(result)) {
      return result;
    } else {
      result = this.getComplexProperty(name, result);
    }
  }
  return result;
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.callProperty = function(name, rest) {
  rest = Array.prototype.slice.call(arguments, 1);
  return this.mx_utils_ObjectProxy__item[name].apply(this.mx_utils_ObjectProxy__item, rest);
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.deleteProperty = function(name) {
  var /** @type {mx.core.IPropertyChangeNotifier} */ notifier = org.apache.royale.utils.Language.as(this.notifiers[name], mx.core.IPropertyChangeNotifier, true);
  if (notifier) {
    notifier.removeEventListener(mx.events.PropertyChangeEvent.PROPERTY_CHANGE, org.apache.royale.utils.Language.closure(this.propertyChangeHandler, this, 'propertyChangeHandler'));
    delete this.notifiers[name];
  }
  var /** @type {*} */ oldVal = this.mx_utils_ObjectProxy__item[name];
  var /** @type {boolean} */ deleted = delete this.mx_utils_ObjectProxy__item[name];
  if (this.dispatcher.hasEventListener(mx.events.PropertyChangeEvent.PROPERTY_CHANGE)) {
    var /** @type {mx.events.PropertyChangeEvent} */ event = new mx.events.PropertyChangeEvent(mx.events.PropertyChangeEvent.PROPERTY_CHANGE);
    event.kind = mx.events.PropertyChangeEventKind.DELETE;
    event.property = name;
    event.oldValue = oldVal;
    event.source = this;
    this.dispatcher.dispatchEvent(event);
  }
  return deleted;
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.hasProperty = function(name) {
  return (name in this.mx_utils_ObjectProxy__item);
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.setProperty = function(name, value) {
  var /** @type {*} */ oldVal = this.mx_utils_ObjectProxy__item[name];
  if (oldVal !== value) {
    this.mx_utils_ObjectProxy__item[name] = value;
    var /** @type {mx.core.IPropertyChangeNotifier} */ notifier = org.apache.royale.utils.Language.as(this.notifiers[name], mx.core.IPropertyChangeNotifier, true);
    if (notifier) {
      notifier.removeEventListener(mx.events.PropertyChangeEvent.PROPERTY_CHANGE, org.apache.royale.utils.Language.closure(this.propertyChangeHandler, this, 'propertyChangeHandler'));
      delete this.notifiers[name];
    }
    if (this.dispatcher.hasEventListener(mx.events.PropertyChangeEvent.PROPERTY_CHANGE)) {
      if (org.apache.royale.utils.Language.is(name, QName))
        name = org.apache.royale.utils.Language.as(name, QName, true).localName;
      var /** @type {mx.events.PropertyChangeEvent} */ event = mx.events.PropertyChangeEvent.createUpdateEvent(this, name.toString(), oldVal, value);
      this.dispatcher.dispatchEvent(event);
    }
  }
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.propertyNames = function() {
  return this.mx_utils_ObjectProxy__item;
};


/**
 * @param {string} name
 * @param {*} value
 * @return {*}
 */
mx.utils.ObjectProxy.prototype.getComplexProperty = function(name, value) {
  if (org.apache.royale.utils.Language.is(value, mx.core.IPropertyChangeNotifier)) {
    value.addEventListener(mx.events.PropertyChangeEvent.PROPERTY_CHANGE, org.apache.royale.utils.Language.closure(this.propertyChangeHandler, this, 'propertyChangeHandler'));
    this.notifiers[name] = value;
    return value;
  }
  if (org.apache.royale.reflection.getQualifiedClassName(value) == "Object") {
    value = new this.proxyClass(this.mx_utils_ObjectProxy__item[name], null, this.mx_utils_ObjectProxy__proxyLevel > 0 ? this.mx_utils_ObjectProxy__proxyLevel - 1 : this.mx_utils_ObjectProxy__proxyLevel);
    value.addEventListener(mx.events.PropertyChangeEvent.PROPERTY_CHANGE, org.apache.royale.utils.Language.closure(this.propertyChangeHandler, this, 'propertyChangeHandler'));
    this.notifiers[name] = value;
    return value;
  }
  return value;
};


/**
 *  Since Flex only uses ObjectProxy to wrap anonymous objects,
 *  the server flex.messaging.io.ObjectProxy instance serializes itself
 *  as a Map that will be returned as a plain ActionScript object. 
 *  You can then set the object_proxy object property to this value.
 *
 *  @asparam input The source object from which the ObjectProxy is
 *  deserialized. 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {org.apache.royale.utils.net.IDataInput} input
 */
mx.utils.ObjectProxy.prototype.readExternal = function(input) {
  var /** @type {Object} */ value = input.readObject();
  this.mx_utils_ObjectProxy__item = value;
};


/**
 *  Since Flex only serializes the inner ActionScript object that it wraps,
 *  the server flex.messaging.io.ObjectProxy populates itself
 *  with this anonymous object's contents and appears to the user
 *  as a Map.
 *
 *  @asparam output The source object from which the ObjectProxy is
 *  deserialized.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {org.apache.royale.utils.net.IDataOutput} output
 */
mx.utils.ObjectProxy.prototype.writeExternal = function(output) {
  output.writeObject(this.mx_utils_ObjectProxy__item);
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.addEventListener = function(type, listener, useCapture, listenerScope) {
  useCapture = typeof useCapture !== 'undefined' ? useCapture : false;
  listenerScope = typeof listenerScope !== 'undefined' ? listenerScope : null;
  this.dispatcher.addEventListener(type, listener, useCapture, listenerScope);
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.removeEventListener = function(type, listener, useCapture, listenerScope) {
  useCapture = typeof useCapture !== 'undefined' ? useCapture : false;
  listenerScope = typeof listenerScope !== 'undefined' ? listenerScope : null;
  this.dispatcher.removeEventListener(type, listener, useCapture, listenerScope);
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.dispatchEvent = function(event) {
  return this.dispatcher.dispatchEvent(event);
};


/**
 * @override
 */
mx.utils.ObjectProxy.prototype.hasEventListener = function(type) {
  return this.dispatcher.hasEventListener(type);
};


/**
 *  Called when a complex property is updated.
 *
 *  @asparam event An event object that has changed.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @param {mx.events.PropertyChangeEvent} event
 */
mx.utils.ObjectProxy.prototype.propertyChangeHandler = function(event) {
  this.dispatcher.dispatchEvent(event);
};


/**
 * @nocollapse
 * @export
 * @type {Object}
 */
mx.utils.ObjectProxy.prototype.object;


mx.utils.ObjectProxy.prototype.get__object = function() {
  return this.mx_utils_ObjectProxy__item;
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.utils.ObjectProxy.prototype.uid;


mx.utils.ObjectProxy.prototype.get__uid = function() {
  if (this.mx_utils_ObjectProxy__id === null)
    this.mx_utils_ObjectProxy__id = mx.utils.UIDUtil.createUID();
  return this.mx_utils_ObjectProxy__id;
};


mx.utils.ObjectProxy.prototype.set__uid = function(value) {
  this.mx_utils_ObjectProxy__id = value;
};


Object.defineProperties(mx.utils.ObjectProxy.prototype, /** @lends {mx.utils.ObjectProxy.prototype} */ {
/**
 * @type {Object}
 */
object: {
get: mx.utils.ObjectProxy.prototype.get__object},
/**
 * @type {string}
 */
uid: {
get: mx.utils.ObjectProxy.prototype.get__uid,
set: mx.utils.ObjectProxy.prototype.set__uid}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.utils.ObjectProxy.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'ObjectProxy', qName: 'mx.utils.ObjectProxy', kind: 'class', isDynamic: true}], interfaces: [org.apache.royale.utils.net.IExternalizable, mx.core.IPropertyChangeNotifier] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.utils.ObjectProxy.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    accessors: function () {
      return {
        'object': { type: 'Object', access: 'readonly', declaredBy: 'mx.utils.ObjectProxy'},
        'uid': { type: 'String', access: 'readwrite', declaredBy: 'mx.utils.ObjectProxy'}
      };
    },
    methods: function () {
      return {
        'ObjectProxy': { type: '', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'Object', true ,'String', true ,'int', true ]; }},
        'getProperty': { type: '*', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ]; }},
        'callProperty': { type: '*', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ '*', false ,'Array', false ]; }},
        'deleteProperty': { type: 'Boolean', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ]; }},
        'hasProperty': { type: 'Boolean', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ]; }},
        'setProperty': { type: 'void', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ,'*', false ]; }},
        'propertyNames': { type: 'Object', declaredBy: 'mx.utils.ObjectProxy'},
        'getComplexProperty': { type: '*', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ,'*', false ]; }},
        'readExternal': { type: 'void', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'org.apache.royale.utils.net.IDataInput', false ]; }},
        'writeExternal': { type: 'void', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'org.apache.royale.utils.net.IDataOutput', false ]; }},
        'addEventListener': { type: 'void', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ,'Function', false ,'Boolean', true ,'Object', true ]; }},
        'removeEventListener': { type: 'void', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ,'Function', false ,'Boolean', true ,'Object', true ]; }},
        'dispatchEvent': { type: 'Boolean', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'org.apache.royale.events.Event', false ]; }},
        'hasEventListener': { type: 'Boolean', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'String', false ]; }},
        'propertyChangeHandler': { type: 'void', declaredBy: 'mx.utils.ObjectProxy', parameters: function () { return [ 'mx.events.PropertyChangeEvent', false ]; }}
      };
    },
    metadata: function () { return [ { name: 'Bindable', args: [ { key: '', value: 'propertyChange' } ] } ]; }
  };
};
/**
 * @const
 * @type {number}
 */
mx.utils.ObjectProxy.prototype.ROYALE_COMPILE_FLAGS = 10;