/**
 * Generated by Apache Royale Compiler from mx/validators/EmailValidator.as
 * mx.validators.EmailValidator
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.validators.EmailValidator');

goog.require('mx.validators.ValidationResult');
goog.require('mx.validators.Validator');
goog.require('XML');



/**
 *  Constructor.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Royale 0.9.3
 * @constructor
 * @extends {mx.validators.Validator}
 */
mx.validators.EmailValidator = function() {
  mx.validators.EmailValidator.base(this, 'constructor');
};
goog.inherits(mx.validators.EmailValidator, mx.validators.Validator);


/**
 * @private
 * @const
 * @type {string}
 */
mx.validators.EmailValidator.DISALLOWED_LOCALNAME_CHARS = "()<>,;:\\\"[] `~!#$%^&*={}|/?\t\n\r";


/**
 * @private
 * @const
 * @type {string}
 */
mx.validators.EmailValidator.DISALLOWED_DOMAIN_CHARS = "()<>,;:\\\"[] `~!#$%^&*+={}|/?'\t\n\r";


/**
 *  Convenience method for calling a validator
 *  from within a custom validation function.
 *  Each of the standard Flex validators has a similar convenience method.
 *
 *  @asparam validator The EmailValidator instance.
 *
 *  @asparam value A field to validate.
 *
 *  @asparam baseField Text representation of the subfield
 *  specified in the value parameter.
 *  For example, if the <code>value</code> parameter specifies value.email,
 *  the <code>baseField</code> value is "email".
 *
 *  @asreturn An Array of ValidationResult objects, with one
 *  ValidationResult object for each field examined by the validator. 
 *
 *  @see mx.validators.ValidationResult	 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {mx.validators.EmailValidator} validator
 * @param {Object} value
 * @param {string} baseField
 * @return {Array}
 */
mx.validators.EmailValidator.validateEmail = function(validator, value, baseField) {
  var /** @type {number} */ i = 0;
  var /** @type {number} */ n = 0;
  var /** @type {Array} */ results = [];
  var /** @type {string} */ emailStr = String(value);
  var /** @type {string} */ username = "";
  var /** @type {string} */ domain = "";
  //var /** @type {number} */ n = 0;
  //var /** @type {number} */ i = 0;
  var /** @type {number} */ ampPos = (emailStr.indexOf("@")) >> 0;
  if (ampPos == -1) {
    results.push(new mx.validators.ValidationResult(true, baseField, "missingAtSign", validator.missingAtSignError));
    return results;
  } else if (emailStr.indexOf("@", ampPos + 1) != -1) {
    results.push(new mx.validators.ValidationResult(true, baseField, "tooManyAtSigns", validator.tooManyAtSignsError));
    return results;
  }
  username = emailStr.substring(0, ampPos);
  domain = emailStr.substring(ampPos + 1);
  var /** @type {number} */ usernameLen = (username.length) >> 0;
  if (usernameLen == 0) {
    results.push(new mx.validators.ValidationResult(true, baseField, "missingUsername", validator.missingUsernameError));
    return results;
  }
  for (i = 0; i < usernameLen; i++) {
    if (mx.validators.EmailValidator.DISALLOWED_LOCALNAME_CHARS.indexOf(username.charAt(i)) != -1) {
      results.push(new mx.validators.ValidationResult(true, baseField, "invalidChar", validator.invalidCharError));
      return results;
    }
  }
  if (username.charAt(0) == '.') {
    results.push(new mx.validators.ValidationResult(true, baseField, "invalidChar", validator.invalidCharError));
    return results;
  }
  var /** @type {number} */ domainLen = (domain.length) >> 0;
  if ((domain.charAt(0) == "[") && (domain.charAt(domainLen - 1) == "]")) {
    if (!mx.validators.EmailValidator.isValidIPAddress(domain.substring(1, domainLen - 1))) {
      results.push(new mx.validators.ValidationResult(true, baseField, "invalidIPDomain", validator.invalidIPDomainError));
      return results;
    }
  } else {
    var /** @type {number} */ periodPos = (domain.indexOf(".")) >> 0;
    var /** @type {number} */ nextPeriodPos = 0;
    var /** @type {string} */ lastDomain = "";
    if (periodPos == -1) {
      results.push(new mx.validators.ValidationResult(true, baseField, "missingPeriodInDomain", validator.missingPeriodInDomainError));
      return results;
    }
    while (true) {
      nextPeriodPos = (domain.indexOf(".", periodPos + 1)) >> 0;
      if (nextPeriodPos == -1) {
        lastDomain = domain.substring(periodPos + 1);
        break;
      } else if (nextPeriodPos == periodPos + 1) {
        results.push(new mx.validators.ValidationResult(true, baseField, "invalidPeriodsInDomain", validator.invalidPeriodsInDomainError));
        return results;
      }
      periodPos = nextPeriodPos;
    }
    for (i = 0; i < domainLen; i++) {
      if (mx.validators.EmailValidator.DISALLOWED_DOMAIN_CHARS.indexOf(domain.charAt(i)) != -1) {
        results.push(new mx.validators.ValidationResult(true, baseField, "invalidChar", validator.invalidCharError));
        return results;
      }
    }
    if (domain.charAt(0) == "." || domain.charAt(0) == "-" || domain.charAt(periodPos - 1) == "-") {
      results.push(new mx.validators.ValidationResult(true, baseField, "invalidDomain", validator.invalidDomainError));
      return results;
    }
  }
  return results;
};


/**
 * Validate a given IP address
 * 
 * If IP domain, then must follow [x.x.x.x] format
 * or for IPv6, then follow [x:x:x:x:x:x:x:x] or [x::x:x:x] or some
 * IPv4 hybrid, like [::x.x.x.x] or [0:00::192.168.0.1]
 *
 * @asprivate
 * @private
 * @param {string} ipAddr
 * @return {boolean}
 */
mx.validators.EmailValidator.isValidIPAddress = function(ipAddr) {
  var /** @type {number} */ i = 0;
  var /** @type {number} */ n = 0;
  var /** @type {Array} */ ipArray = [];
  var /** @type {number} */ pos = 0;
  var /** @type {number} */ newpos = 0;
  var /** @type {number} */ item;
  //var /** @type {number} */ n = 0;
  //var /** @type {number} */ i = 0;
  if (ipAddr.indexOf(":") != -1) {
    var /** @type {boolean} */ hasUnlimitedZeros = ipAddr.indexOf("::") != -1;
    if (hasUnlimitedZeros) {
      ipAddr = ipAddr.replace(/^::/, "");
      ipAddr = ipAddr.replace(/::/g, ":");
    }
    while (true) {
      newpos = (ipAddr.indexOf(":", pos)) >> 0;
      if (newpos != -1) {
        ipArray.push(ipAddr.substring(pos, newpos));
      } else {
        ipArray.push(ipAddr.substring(pos));
        break;
      }
      pos = (newpos + 1) >> 0;
    }
    n = (ipArray.length) >> 0;
    
/**
 * @const
 * @type {boolean}
 */
var lastIsV4 = ipArray[n - 1].indexOf(".") != -1;
    if (lastIsV4) {
      if ((ipArray.length != 7 && !hasUnlimitedZeros) || (ipArray.length > 7))
        return false;
      for (i = 0; i < n; i++) {
        if (i == n - 1) {
          return mx.validators.EmailValidator.isValidIPAddress(org.apache.royale.utils.Language.string(ipArray[i]));
        }
        item = parseInt(ipArray[i], 16);
        if (item != 0)
          return false;
      }
    } else {
      if ((ipArray.length != 8 && !hasUnlimitedZeros) || (ipArray.length > 8))
        return false;
      for (i = 0; i < n; i++) {
        item = parseInt(ipArray[i], 16);
        if (isNaN(item) || item < 0 || item > 0xFFFF || ipArray[i] == "")
          return false;
      }
    }
    return true;
  }
  if (ipAddr.indexOf(".") != -1) {
    while (true) {
      newpos = (ipAddr.indexOf(".", pos)) >> 0;
      if (newpos != -1) {
        ipArray.push(ipAddr.substring(pos, newpos));
      } else {
        ipArray.push(ipAddr.substring(pos));
        break;
      }
      pos = (newpos + 1) >> 0;
    }
    if (ipArray.length != 4)
      return false;
    n = (ipArray.length) >> 0;
    for (i = 0; i < n; i++) {
      item = Number(ipArray[i]);
      if (isNaN(item) || item < 0 || item > 255 || ipArray[i] == "")
        return false;
    }
    return true;
  }
  return false;
};


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__invalidCharError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_invalidCharErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__invalidDomainError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_invalidDomainErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__invalidIPDomainError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_invalidIPDomainErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__invalidPeriodsInDomainError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_invalidPeriodsInDomainErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__missingAtSignError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_missingAtSignErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__missingPeriodInDomainError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_missingPeriodInDomainErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__missingUsernameError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_missingUsernameErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator__tooManyAtSignsError;


/**
 * @private
 * @type {string}
 */
mx.validators.EmailValidator.prototype.mx_validators_EmailValidator_tooManyAtSignsErrorOverride;


/**
 *  Override of the base class <code>doValidation()</code> method
 *  to validate an e-mail address.
 *
 *  <p>You do not call this method directly;
 *  Flex calls it as part of performing a validation.
 *  If you create a custom Validator class, you must implement this method. </p>
 *
 *  @asparam value Either a String or an Object to validate.
 *
 *  @asreturn An Array of ValidationResult objects, with one ValidationResult 
 *  object for each field examined by the validator. 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @protected
 * @override
 */
mx.validators.EmailValidator.prototype.doValidation = function(value) {
  var /** @type {Array} */ results = mx.validators.EmailValidator.superClass_.doValidation.apply(this, [value]);
  var /** @type {string} */ val = value ? String(value) : "";
  if (results.length > 0 || ((val.length == 0) && !this.required))
    return results;
  else
    return mx.validators.EmailValidator.validateEmail(this, value, null);
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.invalidCharError;


mx.validators.EmailValidator.prototype.get__invalidCharError = function() {
  return this.mx_validators_EmailValidator__invalidCharError;
};


mx.validators.EmailValidator.prototype.set__invalidCharError = function(value) {
  this.mx_validators_EmailValidator_invalidCharErrorOverride = value;
  this.mx_validators_EmailValidator__invalidCharError = value != null ? value : "invalidCharErrorEV";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.invalidDomainError;


mx.validators.EmailValidator.prototype.get__invalidDomainError = function() {
  return this.mx_validators_EmailValidator__invalidDomainError;
};


mx.validators.EmailValidator.prototype.set__invalidDomainError = function(value) {
  this.mx_validators_EmailValidator_invalidDomainErrorOverride = value;
  this.mx_validators_EmailValidator__invalidDomainError = value != null ? value : "invalidDomainErrorEV";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.invalidIPDomainError;


mx.validators.EmailValidator.prototype.get__invalidIPDomainError = function() {
  return this.mx_validators_EmailValidator__invalidIPDomainError;
};


mx.validators.EmailValidator.prototype.set__invalidIPDomainError = function(value) {
  this.mx_validators_EmailValidator_invalidIPDomainErrorOverride = value;
  this.mx_validators_EmailValidator__invalidIPDomainError = value != null ? value : "invalidIPDomainError";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.invalidPeriodsInDomainError;


mx.validators.EmailValidator.prototype.get__invalidPeriodsInDomainError = function() {
  return this.mx_validators_EmailValidator__invalidPeriodsInDomainError;
};


mx.validators.EmailValidator.prototype.set__invalidPeriodsInDomainError = function(value) {
  this.mx_validators_EmailValidator_invalidPeriodsInDomainErrorOverride = value;
  this.mx_validators_EmailValidator__invalidPeriodsInDomainError = value != null ? value : "invalidPeriodsInDomainError";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.missingAtSignError;


mx.validators.EmailValidator.prototype.get__missingAtSignError = function() {
  return this.mx_validators_EmailValidator__missingAtSignError;
};


mx.validators.EmailValidator.prototype.set__missingAtSignError = function(value) {
  this.mx_validators_EmailValidator_missingAtSignErrorOverride = value;
  this.mx_validators_EmailValidator__missingAtSignError = value != null ? value : "missingAtSignError";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.missingPeriodInDomainError;


mx.validators.EmailValidator.prototype.get__missingPeriodInDomainError = function() {
  return this.mx_validators_EmailValidator__missingPeriodInDomainError;
};


mx.validators.EmailValidator.prototype.set__missingPeriodInDomainError = function(value) {
  this.mx_validators_EmailValidator_missingPeriodInDomainErrorOverride = value;
  this.mx_validators_EmailValidator__missingPeriodInDomainError = value != null ? value : "missingPeriodInDomainError";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.missingUsernameError;


mx.validators.EmailValidator.prototype.get__missingUsernameError = function() {
  return this.mx_validators_EmailValidator__missingUsernameError;
};


mx.validators.EmailValidator.prototype.set__missingUsernameError = function(value) {
  this.mx_validators_EmailValidator_missingUsernameErrorOverride = value;
  this.mx_validators_EmailValidator__missingUsernameError = value != null ? value : "missingUsernameError";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.EmailValidator.prototype.tooManyAtSignsError;


mx.validators.EmailValidator.prototype.get__tooManyAtSignsError = function() {
  return this.mx_validators_EmailValidator__tooManyAtSignsError;
};


mx.validators.EmailValidator.prototype.set__tooManyAtSignsError = function(value) {
  this.mx_validators_EmailValidator_tooManyAtSignsErrorOverride = value;
  this.mx_validators_EmailValidator__tooManyAtSignsError = value != null ? value : "tooManyAtSignsError";
};


Object.defineProperties(mx.validators.EmailValidator.prototype, /** @lends {mx.validators.EmailValidator.prototype} */ {
/**
 * @type {string}
 */
invalidCharError: {
get: mx.validators.EmailValidator.prototype.get__invalidCharError,
set: mx.validators.EmailValidator.prototype.set__invalidCharError},
/**
 * @type {string}
 */
invalidDomainError: {
get: mx.validators.EmailValidator.prototype.get__invalidDomainError,
set: mx.validators.EmailValidator.prototype.set__invalidDomainError},
/**
 * @type {string}
 */
invalidIPDomainError: {
get: mx.validators.EmailValidator.prototype.get__invalidIPDomainError,
set: mx.validators.EmailValidator.prototype.set__invalidIPDomainError},
/**
 * @type {string}
 */
invalidPeriodsInDomainError: {
get: mx.validators.EmailValidator.prototype.get__invalidPeriodsInDomainError,
set: mx.validators.EmailValidator.prototype.set__invalidPeriodsInDomainError},
/**
 * @type {string}
 */
missingAtSignError: {
get: mx.validators.EmailValidator.prototype.get__missingAtSignError,
set: mx.validators.EmailValidator.prototype.set__missingAtSignError},
/**
 * @type {string}
 */
missingPeriodInDomainError: {
get: mx.validators.EmailValidator.prototype.get__missingPeriodInDomainError,
set: mx.validators.EmailValidator.prototype.set__missingPeriodInDomainError},
/**
 * @type {string}
 */
missingUsernameError: {
get: mx.validators.EmailValidator.prototype.get__missingUsernameError,
set: mx.validators.EmailValidator.prototype.set__missingUsernameError},
/**
 * @type {string}
 */
tooManyAtSignsError: {
get: mx.validators.EmailValidator.prototype.get__tooManyAtSignsError,
set: mx.validators.EmailValidator.prototype.set__tooManyAtSignsError}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.validators.EmailValidator.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'EmailValidator', qName: 'mx.validators.EmailValidator', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.validators.EmailValidator.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    accessors: function () {
      return {
        'invalidCharError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'invalidDomainError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'invalidIPDomainError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'invalidPeriodsInDomainError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'missingAtSignError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'missingPeriodInDomainError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'missingUsernameError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'},
        'tooManyAtSignsError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.EmailValidator'}
      };
    },
    methods: function () {
      return {
        '|validateEmail': { type: 'Array', declaredBy: 'mx.validators.EmailValidator', parameters: function () { return [ 'mx.validators.EmailValidator', false ,'Object', false ,'String', false ]; }},
        'EmailValidator': { type: '', declaredBy: 'mx.validators.EmailValidator'}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.validators.EmailValidator.prototype.ROYALE_COMPILE_FLAGS = 10;