/**
 * Generated by Apache Royale Compiler from mx/validators/StringValidator.as
 * mx.validators.StringValidator
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.validators.StringValidator');

goog.require('mx.utils.StringUtil');
goog.require('mx.validators.ValidationResult');
goog.require('mx.validators.Validator');
goog.require('XML');



/**
 *  Constructor.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @extends {mx.validators.Validator}
 */
mx.validators.StringValidator = function() {
  mx.validators.StringValidator.base(this, 'constructor');
};
goog.inherits(mx.validators.StringValidator, mx.validators.Validator);


/**
 *  Convenience method for calling a validator.
 *  Each of the standard Flex validators has a similar convenience method.
 *
 *  @asparam validator The StringValidator instance.
 *
 *  @asparam value A field to validate.
 *
 *  @asparam baseField Text representation of the subfield
 *  specified in the <code>value</code> parameter.
 *  For example, if the <code>value</code> parameter specifies
 *  value.mystring, the <code>baseField</code> value
 *  is <code>"mystring"</code>.
 *
 *  @asreturn An Array of ValidationResult objects, with one
 *  ValidationResult  object for each field examined by the validator. 
 *
 *  @see mx.validators.ValidationResult
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @nocollapse
 * @param {mx.validators.StringValidator} validator
 * @param {Object} value
 * @param {string=} baseField
 * @return {Array}
 */
mx.validators.StringValidator.validateString = function(validator, value, baseField) {
  baseField = typeof baseField !== 'undefined' ? baseField : null;
  var /** @type {Array} */ results = [];
  var /** @type {number} */ maxLength = Number(validator.maxLength);
  var /** @type {number} */ minLength = Number(validator.minLength);
  var /** @type {string} */ val = value != null ? String(value) : "";
  if (!isNaN(maxLength) && val.length > maxLength) {
    results.push(new mx.validators.ValidationResult(true, baseField, "tooLong", mx.utils.StringUtil.substitute(validator.tooLongError, maxLength)));
    return results;
  }
  if (!isNaN(minLength) && val.length < minLength) {
    results.push(new mx.validators.ValidationResult(true, baseField, "tooShort", mx.utils.StringUtil.substitute(validator.tooShortError, minLength)));
    return results;
  }
  return results;
};


/**
 * @private
 * @type {Object}
 */
mx.validators.StringValidator.prototype.mx_validators_StringValidator__maxLength;


/**
 * @private
 * @type {Object}
 */
mx.validators.StringValidator.prototype.mx_validators_StringValidator_maxLengthOverride;


/**
 * @private
 * @type {Object}
 */
mx.validators.StringValidator.prototype.mx_validators_StringValidator__minLength;


/**
 * @private
 * @type {Object}
 */
mx.validators.StringValidator.prototype.mx_validators_StringValidator_minLengthOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.StringValidator.prototype.mx_validators_StringValidator__tooLongError = "This string is longer than the maximum allowed length. This must be less than {0} characters long.";


/**
 * @private
 * @type {string}
 */
mx.validators.StringValidator.prototype.mx_validators_StringValidator__tooShortError = "This string is shorter than the minimum allowed length. This must be at least {0} characters long.";


/**
 *  Override of the base class <code>doValidation()</code> method
 *  to validate a String.
 *
 *  <p>You do not call this method directly;
 *  Flex calls it as part of performing a validation.
 *  If you create a custom Validator class, you must implement this method.</p>
 *
 *  @asparam value Object to validate.
 *
 *  @asreturn An Array of ValidationResult objects, with one ValidationResult 
 *  object for each field examined by the validator. 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @protected
 * @override
 */
mx.validators.StringValidator.prototype.doValidation = function(value) {
  var /** @type {Array} */ results = mx.validators.StringValidator.superClass_.doValidation.apply(this, [value]);
  var /** @type {string} */ val = value ? String(value) : "";
  if (results.length > 0 || ((val.length == 0) && !this.required))
    return results;
  else
    return mx.validators.StringValidator.validateString(this, value, null);
};


/**
 * @nocollapse
 * @export
 * @type {Object}
 */
mx.validators.StringValidator.prototype.maxLength;


mx.validators.StringValidator.prototype.get__maxLength = function() {
  return this.mx_validators_StringValidator__maxLength;
};


mx.validators.StringValidator.prototype.set__maxLength = function(value) {
  this.mx_validators_StringValidator__maxLength = value != null ? Number(value) : 20;
};


/**
 * @nocollapse
 * @export
 * @type {Object}
 */
mx.validators.StringValidator.prototype.minLength;


mx.validators.StringValidator.prototype.get__minLength = function() {
  return this.mx_validators_StringValidator__minLength;
};


mx.validators.StringValidator.prototype.set__minLength = function(value) {
  this.mx_validators_StringValidator__minLength = value != null ? Number(value) : 0;
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.StringValidator.prototype.tooLongError;


mx.validators.StringValidator.prototype.get__tooLongError = function() {
  return this.mx_validators_StringValidator__tooLongError;
};


mx.validators.StringValidator.prototype.set__tooLongError = function(value) {
  this.mx_validators_StringValidator__tooLongError = value != null ? value : "tooLongError";
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.StringValidator.prototype.tooShortError;


mx.validators.StringValidator.prototype.get__tooShortError = function() {
  return this.mx_validators_StringValidator__tooShortError;
};


mx.validators.StringValidator.prototype.set__tooShortError = function(value) {
  this.mx_validators_StringValidator__tooShortError = value != null ? value : "tooShortError";
};


Object.defineProperties(mx.validators.StringValidator.prototype, /** @lends {mx.validators.StringValidator.prototype} */ {
/**
 * @type {Object}
 */
maxLength: {
get: mx.validators.StringValidator.prototype.get__maxLength,
set: mx.validators.StringValidator.prototype.set__maxLength},
/**
 * @type {Object}
 */
minLength: {
get: mx.validators.StringValidator.prototype.get__minLength,
set: mx.validators.StringValidator.prototype.set__minLength},
/**
 * @type {string}
 */
tooLongError: {
get: mx.validators.StringValidator.prototype.get__tooLongError,
set: mx.validators.StringValidator.prototype.set__tooLongError},
/**
 * @type {string}
 */
tooShortError: {
get: mx.validators.StringValidator.prototype.get__tooShortError,
set: mx.validators.StringValidator.prototype.set__tooShortError}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.validators.StringValidator.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'StringValidator', qName: 'mx.validators.StringValidator', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.validators.StringValidator.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    accessors: function () {
      return {
        'maxLength': { type: 'Object', access: 'readwrite', declaredBy: 'mx.validators.StringValidator'},
        'minLength': { type: 'Object', access: 'readwrite', declaredBy: 'mx.validators.StringValidator'},
        'tooLongError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.StringValidator'},
        'tooShortError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.StringValidator'}
      };
    },
    methods: function () {
      return {
        '|validateString': { type: 'Array', declaredBy: 'mx.validators.StringValidator', parameters: function () { return [ 'mx.validators.StringValidator', false ,'Object', false ,'String', true ]; }},
        'StringValidator': { type: '', declaredBy: 'mx.validators.StringValidator'}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.validators.StringValidator.prototype.ROYALE_COMPILE_FLAGS = 10;