/**
 * Generated by Apache Royale Compiler from mx/validators/ZipCodeValidator.as
 * mx.validators.ZipCodeValidator
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('mx.validators.ZipCodeValidator');

goog.require('mx.resources.IResourceManager');
goog.require('mx.resources.ResourceManager');
goog.require('mx.validators.ValidationResult');
goog.require('mx.validators.Validator');
goog.require('mx.validators.ZipCodeValidatorDomainType');
goog.require('XML');



/**
 *  Constructor.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @constructor
 * @extends {mx.validators.Validator}
 */
mx.validators.ZipCodeValidator = function() {
  
  this.resourceManager = mx.resources.ResourceManager.getInstance();
  mx.validators.ZipCodeValidator.base(this, 'constructor');
};
goog.inherits(mx.validators.ZipCodeValidator, mx.validators.Validator);


/**
 * @private
 * @const
 * @type {number}
 */
mx.validators.ZipCodeValidator.DOMAIN_US = 1;


/**
 * @private
 * @const
 * @type {number}
 */
mx.validators.ZipCodeValidator.DOMAIN_US_OR_CANADA = 2;


/**
 * @private
 * @const
 * @type {number}
 */
mx.validators.ZipCodeValidator.DOMAIN_CANADA = 3;


/**
 * @type {mx.resources.IResourceManager}
 */
mx.validators.ZipCodeValidator.prototype.resourceManager;


/**
 * @nocollapse
 * @param {mx.validators.ZipCodeValidator} validator
 * @param {Object} value
 * @param {string} baseField
 * @return {Array}
 */
mx.validators.ZipCodeValidator.validateZipCode = function(validator, value, baseField) {
  var /** @type {number} */ i = 0;
  var /** @type {number} */ n = 0;
  var /** @type {Array} */ results = [];
  var /** @type {string} */ allowedFormatChars = validator.allowedFormatChars;
  var /** @type {string} */ domain = validator.domain;
  var /** @type {mx.resources.IResourceManager} */ resourceManager = mx.resources.ResourceManager.getInstance();
  var /** @type {string} */ zip = String(value);
  var /** @type {number} */ len = (zip.length) >> 0;
  var /** @type {number} */ domainType = mx.validators.ZipCodeValidator.DOMAIN_US;
  switch (domain) {
    case mx.validators.ZipCodeValidatorDomainType.US_OR_CANADA:
      domainType = mx.validators.ZipCodeValidator.DOMAIN_US_OR_CANADA;
      break;
    case mx.validators.ZipCodeValidatorDomainType.US_ONLY:
      domainType = mx.validators.ZipCodeValidator.DOMAIN_US;
      break;
    case mx.validators.ZipCodeValidatorDomainType.CANADA_ONLY:
      domainType = mx.validators.ZipCodeValidator.DOMAIN_CANADA;
      break;
    default:
      results.push(new mx.validators.ValidationResult(true, baseField, "invalidDomain", validator.invalidDomainError));
      return results;
  }
  //var /** @type {number} */ n = 0;
  //var /** @type {number} */ i = 0;
  var /** @type {string} */ c;
  n = (allowedFormatChars.length) >> 0;
  for (i = 0; i < n; i++) {
    c = allowedFormatChars.charAt(i);
    if (mx.validators.Validator.DECIMAL_DIGITS.indexOf(c) != -1 || mx.validators.Validator.ROMAN_LETTERS.indexOf(c) != -1) {
      var /** @type {string} */ message = resourceManager.getString("validators", "invalidFormatCharsZCV");
      throw new Error(message);
    }
  }
  for (i = 0; i < len; i++) {
    c = zip.charAt(i);
    if (mx.validators.Validator.ROMAN_LETTERS.indexOf(c) == -1 && mx.validators.Validator.DECIMAL_DIGITS.indexOf(c) == -1 && allowedFormatChars.indexOf(c) == -1) {
      results.push(new mx.validators.ValidationResult(true, baseField, "invalidChar", validator.invalidCharError));
      return results;
    }
  }
  var /** @type {boolean} */ containsLetters = false;
  for (i = 0; i < len; i++) {
    if (mx.validators.Validator.ROMAN_LETTERS.indexOf(zip.charAt(i)) != -1) {
      containsLetters = true;
      break;
    }
  }
  if ((len < 5 || len > 10) || (len == 8) || (!containsLetters && (len == 6 || len == 7))) {
    results.push(new mx.validators.ValidationResult(true, baseField, "wrongLength", validator.wrongLengthError));
    return results;
  }
  switch (domainType) {
    case mx.validators.ZipCodeValidator.DOMAIN_US:
      if (validator.mx_validators_ZipCodeValidator_validateUSCode(zip, containsLetters) == false) {
        results.push(new mx.validators.ValidationResult(true, baseField, "wrongUSFormat", validator.wrongUSFormatError));
        return results;
      }
      break;
    case mx.validators.ZipCodeValidator.DOMAIN_CANADA:
      if (validator.mx_validators_ZipCodeValidator_validateCACode(zip, containsLetters) == false) {
        results.push(new mx.validators.ValidationResult(true, baseField, "wrongCAFormat", validator.wrongCAFormatError));
        return results;
      }
      break;
    case mx.validators.ZipCodeValidator.DOMAIN_US_OR_CANADA:
      var /** @type {boolean} */ valid = true;
      var /** @type {mx.validators.ValidationResult} */ validationResult;
      if (len == 5 || len == 9 || len == 10) {
        if (validator.mx_validators_ZipCodeValidator_validateUSCode(zip, containsLetters) == false) {
          validationResult = new mx.validators.ValidationResult(true, baseField, "wrongUSFormat", validator.wrongUSFormatError);
          valid = false;
        }
      } else {
        if (validator.mx_validators_ZipCodeValidator_validateCACode(zip, containsLetters) == false) {
          validationResult = new mx.validators.ValidationResult(true, baseField, "wrongCAFormat", validator.wrongCAFormatError);
          valid = false;
        }
      }
      if (!valid) {
        results.push(validationResult);
        return results;
      }
      break;
  }
  return results;
};


/**
 *  @asprivate
 * @private
 * @param {string} zip
 * @param {boolean} containsLetters
 * @return {boolean}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_validateUSCode = function(zip, containsLetters) {
  var /** @type {number} */ i = 0;
  var /** @type {number} */ len = (zip.length) >> 0;
  if (containsLetters) {
    return false;
  }
  //var /** @type {number} */ i = 0;
  for (i = 0; i < 5; i++) {
    if (mx.validators.Validator.DECIMAL_DIGITS.indexOf(zip.charAt(i)) == -1) {
      return false;
    }
  }
  if (len == 9 || len == 10) {
    if (len == 10) {
      if (this.allowedFormatChars.indexOf(zip.charAt(5)) == -1) {
        return false;
      }
      i++;
    }
    for (; i < len; i++) {
      if (mx.validators.Validator.DECIMAL_DIGITS.indexOf(zip.charAt(i)) == -1) {
        return false;
      }
    }
  }
  return true;
};


/**
 *  @asprivate
 * @private
 * @param {string} zip
 * @param {boolean} containsLetters
 * @return {boolean}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_validateCACode = function(zip, containsLetters) {
  var /** @type {number} */ len = (zip.length) >> 0;
  if (!containsLetters) {
    return false;
  }
  var /** @type {number} */ i = 0;
  if (mx.validators.Validator.ROMAN_LETTERS.indexOf(zip.charAt(i++)) == -1 || mx.validators.Validator.DECIMAL_DIGITS.indexOf(zip.charAt(i++)) == -1 || mx.validators.Validator.ROMAN_LETTERS.indexOf(zip.charAt(i++)) == -1) {
    return false;
  }
  if (len == 7 && this.allowedFormatChars.indexOf(zip.charAt(i++)) == -1) {
    return false;
  }
  if (mx.validators.Validator.DECIMAL_DIGITS.indexOf(zip.charAt(i++)) == -1 || mx.validators.Validator.ROMAN_LETTERS.indexOf(zip.charAt(i++)) == -1 || mx.validators.Validator.DECIMAL_DIGITS.indexOf(zip.charAt(i++)) == -1) {
    return false;
  }
  return true;
};


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__allowedFormatChars;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_allowedFormatCharsOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__domain;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_domainOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__invalidCharError;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_invalidCharErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__invalidDomainError;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_invalidDomainErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__wrongCAFormatError;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_wrongCAFormatErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__wrongLengthError;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_wrongLengthErrorOverride;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator__wrongUSFormatError;


/**
 * @private
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.mx_validators_ZipCodeValidator_wrongUSFormatErrorOverride;


/**
 *  @asprivate    
 * @protected
 * @override
 */
mx.validators.ZipCodeValidator.prototype.resourcesChanged = function() {
  mx.validators.ZipCodeValidator.superClass_.resourcesChanged.apply(this);
  this.allowedFormatChars = this.mx_validators_ZipCodeValidator_allowedFormatCharsOverride;
  this.domain = this.mx_validators_ZipCodeValidator_domainOverride;
  this.invalidDomainError = this.mx_validators_ZipCodeValidator_invalidDomainErrorOverride;
  this.invalidCharError = this.mx_validators_ZipCodeValidator_invalidCharErrorOverride;
  this.wrongCAFormatError = this.mx_validators_ZipCodeValidator_wrongCAFormatErrorOverride;
  this.wrongLengthError = this.mx_validators_ZipCodeValidator_wrongLengthErrorOverride;
  this.wrongUSFormatError = this.mx_validators_ZipCodeValidator_wrongUSFormatErrorOverride;
};


/**
 *  Override of the base class <code>doValidation()</code> method
 *  to validate a ZIP code.
 *
 *  <p>You do not call this method directly;
 *  Flex calls it as part of performing a validation.
 *  If you create a custom Validator class, you must implement this method. </p>
 *
 *  @asparam value Object to validate.
 *
 *  @asreturn An Array of ValidationResult objects, with one ValidationResult 
 *  object for each field examined by the validator. 
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 * @protected
 * @override
 */
mx.validators.ZipCodeValidator.prototype.doValidation = function(value) {
  var /** @type {Array} */ results = mx.validators.ZipCodeValidator.superClass_.doValidation.apply(this, [value]);
  var /** @type {string} */ val = value ? String(value) : "";
  if (results.length > 0 || ((val.length == 0) && !this.required))
    return results;
  else
    return mx.validators.ZipCodeValidator.validateZipCode(this, value, null);
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.allowedFormatChars;


mx.validators.ZipCodeValidator.prototype.get__allowedFormatChars = function() {
  return this.mx_validators_ZipCodeValidator__allowedFormatChars;
};


mx.validators.ZipCodeValidator.prototype.set__allowedFormatChars = function(value) {
  if (value != null) {
    for (var /** @type {number} */ i = 0; i < value.length; i++) {
      var /** @type {string} */ c = value.charAt(i);
      if (mx.validators.Validator.DECIMAL_DIGITS.indexOf(c) != -1 || mx.validators.Validator.ROMAN_LETTERS.indexOf(c) != -1) {
        var /** @type {string} */ message = this.resourceManager.getString("validators", "invalidFormatCharsZCV");
        throw new Error(message);
      }
    }
  }
  this.mx_validators_ZipCodeValidator_allowedFormatCharsOverride = value;
  this.mx_validators_ZipCodeValidator__allowedFormatChars = value != null ? value : this.resourceManager.getString("validators", "zipCodeValidatorAllowedFormatChars");
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.domain;


mx.validators.ZipCodeValidator.prototype.get__domain = function() {
  return this.mx_validators_ZipCodeValidator__domain;
};


mx.validators.ZipCodeValidator.prototype.set__domain = function(value) {
  this.mx_validators_ZipCodeValidator_domainOverride = value;
  this.mx_validators_ZipCodeValidator__domain = value != null ? value : this.resourceManager.getString("validators", "zipCodeValidatorDomain");
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.invalidCharError;


mx.validators.ZipCodeValidator.prototype.get__invalidCharError = function() {
  return this.mx_validators_ZipCodeValidator__invalidCharError;
};


mx.validators.ZipCodeValidator.prototype.set__invalidCharError = function(value) {
  this.mx_validators_ZipCodeValidator_invalidCharErrorOverride = value;
  this.mx_validators_ZipCodeValidator__invalidCharError = value != null ? value : this.resourceManager.getString("validators", "invalidCharErrorZCV");
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.invalidDomainError;


mx.validators.ZipCodeValidator.prototype.get__invalidDomainError = function() {
  return this.mx_validators_ZipCodeValidator__invalidDomainError;
};


mx.validators.ZipCodeValidator.prototype.set__invalidDomainError = function(value) {
  this.mx_validators_ZipCodeValidator_invalidDomainErrorOverride = value;
  this.mx_validators_ZipCodeValidator__invalidDomainError = value != null ? value : this.resourceManager.getString("validators", "invalidDomainErrorZCV");
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.wrongCAFormatError;


mx.validators.ZipCodeValidator.prototype.get__wrongCAFormatError = function() {
  return this.mx_validators_ZipCodeValidator__wrongCAFormatError;
};


mx.validators.ZipCodeValidator.prototype.set__wrongCAFormatError = function(value) {
  this.mx_validators_ZipCodeValidator_wrongCAFormatErrorOverride = value;
  this.mx_validators_ZipCodeValidator__wrongCAFormatError = value != null ? value : this.resourceManager.getString("validators", "wrongCAFormatError");
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.wrongLengthError;


mx.validators.ZipCodeValidator.prototype.get__wrongLengthError = function() {
  return this.mx_validators_ZipCodeValidator__wrongLengthError;
};


mx.validators.ZipCodeValidator.prototype.set__wrongLengthError = function(value) {
  this.mx_validators_ZipCodeValidator_wrongLengthErrorOverride = value;
  this.mx_validators_ZipCodeValidator__wrongLengthError = value != null ? value : this.resourceManager.getString("validators", "wrongLengthErrorZCV");
};


/**
 * @nocollapse
 * @export
 * @type {string}
 */
mx.validators.ZipCodeValidator.prototype.wrongUSFormatError;


mx.validators.ZipCodeValidator.prototype.get__wrongUSFormatError = function() {
  return this.mx_validators_ZipCodeValidator__wrongUSFormatError;
};


mx.validators.ZipCodeValidator.prototype.set__wrongUSFormatError = function(value) {
  this.mx_validators_ZipCodeValidator_wrongUSFormatErrorOverride = value;
  this.mx_validators_ZipCodeValidator__wrongUSFormatError = value != null ? value : this.resourceManager.getString("validators", "wrongUSFormatError");
};


Object.defineProperties(mx.validators.ZipCodeValidator.prototype, /** @lends {mx.validators.ZipCodeValidator.prototype} */ {
/**
 * @type {string}
 */
allowedFormatChars: {
get: mx.validators.ZipCodeValidator.prototype.get__allowedFormatChars,
set: mx.validators.ZipCodeValidator.prototype.set__allowedFormatChars},
/**
 * @type {string}
 */
domain: {
get: mx.validators.ZipCodeValidator.prototype.get__domain,
set: mx.validators.ZipCodeValidator.prototype.set__domain},
/**
 * @type {string}
 */
invalidCharError: {
get: mx.validators.ZipCodeValidator.prototype.get__invalidCharError,
set: mx.validators.ZipCodeValidator.prototype.set__invalidCharError},
/**
 * @type {string}
 */
invalidDomainError: {
get: mx.validators.ZipCodeValidator.prototype.get__invalidDomainError,
set: mx.validators.ZipCodeValidator.prototype.set__invalidDomainError},
/**
 * @type {string}
 */
wrongCAFormatError: {
get: mx.validators.ZipCodeValidator.prototype.get__wrongCAFormatError,
set: mx.validators.ZipCodeValidator.prototype.set__wrongCAFormatError},
/**
 * @type {string}
 */
wrongLengthError: {
get: mx.validators.ZipCodeValidator.prototype.get__wrongLengthError,
set: mx.validators.ZipCodeValidator.prototype.set__wrongLengthError},
/**
 * @type {string}
 */
wrongUSFormatError: {
get: mx.validators.ZipCodeValidator.prototype.get__wrongUSFormatError,
set: mx.validators.ZipCodeValidator.prototype.set__wrongUSFormatError}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
mx.validators.ZipCodeValidator.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'ZipCodeValidator', qName: 'mx.validators.ZipCodeValidator', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
mx.validators.ZipCodeValidator.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'resourceManager': { type: 'mx.resources.IResourceManager', get_set: function (/** mx.validators.ZipCodeValidator */ inst, /** * */ v) {return v !== undefined ? inst.resourceManager = v : inst.resourceManager;}}
      };
    },
    accessors: function () {
      return {
        'allowedFormatChars': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'},
        'domain': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'},
        'invalidCharError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'},
        'invalidDomainError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'},
        'wrongCAFormatError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'},
        'wrongLengthError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'},
        'wrongUSFormatError': { type: 'String', access: 'readwrite', declaredBy: 'mx.validators.ZipCodeValidator'}
      };
    },
    methods: function () {
      return {
        '|validateZipCode': { type: 'Array', declaredBy: 'mx.validators.ZipCodeValidator', parameters: function () { return [ 'mx.validators.ZipCodeValidator', false ,'Object', false ,'String', false ]; }},
        'ZipCodeValidator': { type: '', declaredBy: 'mx.validators.ZipCodeValidator'}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
mx.validators.ZipCodeValidator.prototype.ROYALE_COMPILE_FLAGS = 10;